﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Ensaios;
using TestView.Ensaios.Utilidades.CorposDeProva;
using TestView.Condicionador;
using TestView.Ensaios.Utilidades;
using ModuloEnsaioTracao.Graficos;
using TestView.Serializacao;
using TestView.Ensaios.Utilidades.Coleta;
using System.Runtime.Serialization;
using System.IO;
using ModuloEnsaioTracao.Ensaios.CorposDeProva;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    [KnownType(typeof(Quadrado))]
    [KnownType(typeof(Cilindrico))]
    [KnownType(typeof(Plano))]
    public class EnsaioTracao : Ensaio
    {
        #region Construtor e membros

        private DateTime _dataEnsaio;
        private EstadoEnsaio _estadoEnsaio;
        private CorpoDeProva _corpoProva;
        private Aquisicao _aquisicao;
        private DateTime _tempoInicioEnsaio;
        private GraficoEnsaioTracao _grafico;

        private List<InformacaoAdicional> _informacoesAdicionais;

        private Double _forcaMaxima;

        public EnsaioTracao()
        {
            this._estadoEnsaio = EstadoEnsaio.NAO_INICIADO;
            this._dataEnsaio = DateTime.Now;
            this._corpoProva = new Cilindrico(1);
            this._aquisicao = new Aquisicao();
            this._forcaMaxima = double.NaN;
            this._informacoesAdicionais = new List<InformacaoAdicional>()
            {
                new InformacaoAdicional("Empresa", ""),
                new InformacaoAdicional("Operador", ""),
                new InformacaoAdicional("Teste1", ""),
                new InformacaoAdicional("Teste2", ""),
            };
            this._grafico = new GraficoEnsaioTracao();
        }

        [DataMember]
        public override EstadoEnsaio EstadoEnsaio
        {
            get
            {
                return this._estadoEnsaio;
            }
            set
            {
                this._estadoEnsaio = value;
                this.informEstadoEnsaioListeners();
            }
        }

        [DataMember]
        public override DateTime DataEnsaio
        {
            get
            {
                return this._dataEnsaio;
            }
            set
            {
                this._dataEnsaio = value;
            }
        }

        [DataMember]
        public override CorpoDeProva CorpoDeProva
        {
            get
            {
                return this._corpoProva;
            }
            set
            {
                this._corpoProva = value;
            }
        }

        [DataMember]
        public Aquisicao Aquisicao
        {
            get
            {
                return this._aquisicao;
            }
            set
            {
                this._aquisicao = value;
            }
        }

        [DataMember]
        public DateTime HoraInicioEnsaio
        {
            get
            {
                return this._tempoInicioEnsaio;
            }
            set
            {
                this._tempoInicioEnsaio = value;
            }
        }

        [DataMember]
        public List<InformacaoAdicional> InformacoesAdicionais
        {
            get
            {
                return this._informacoesAdicionais;
            }
            set
            {
                this._informacoesAdicionais = value;
            }
        }

        [DataMember]
        public GraficoEnsaioTracao Grafico
        {
            get
            {
                return this._grafico;
            }
            set
            {
                this._grafico = value;
            }
        }

        #region Dados do ensaio

        [DataMember]
        public double ForcaMaxima
        {
            get
            {
                return this._forcaMaxima;
            }
            set
            {
                this._forcaMaxima = value;
            }
        }

        #endregion

        #endregion

        #region Serializacao

        public override void serializaEnsaio(Stream stream)
        {
            Serializador.serializaObjeto(stream, this, typeof(EnsaioTracao));
        }

        public override Ensaio deserializaEnsaio(Stream stream)
        {
            Ensaio ensaio = (Ensaio)Serializador.deserializaObjeto(stream, typeof(EnsaioTracao));
            return ensaio;
        }

        #endregion

        #region Misc

        public override string ToString()
        {
            return String.Format("Ensaio de Tração {0:dd-MM-yy HH:mm}", this.DataEnsaio);
        }

        #endregion
    }
}
