﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Ensaios.Utilidades.CorposDeProva;
using TestView.Ensaios.Utilidades;
using ModuloEnsaioTracao.Ensaios.CorposDeProva;
using ModuloEnsaioTracao.Graficos;
using TestView.Ensaios;
using System.IO;
using TestView.Serializacao;
using System.Runtime.Serialization;
using TestView.Seguranca;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    [KnownType(typeof(Quadrado))]
    [KnownType(typeof(Cilindrico))]
    [KnownType(typeof(Plano))]
    public class ModeloDeEnsaioTracao : IModeloDeEnsaio
    {
        #region Construtor e membros
        private static readonly string NOME_ARQUIVO_MODELO = "modelo_ensaio_tracao.xml";

        private static ModeloDeEnsaioTracao _instance;

        private CorpoDeProva _corpoDeProva;
        private OpcoesGraficoTracao _opcoesGraficoTracao;
        private List<InformacaoAdicional> _informacoesAdicionais;

        [DataMember]
        public CorpoDeProva CorpoDeProva
        {
            get
            {
                return this._corpoDeProva;
            }
            set
            {
                this._corpoDeProva = value;
            }
        }
        [DataMember]
        public OpcoesGraficoTracao OpcoesDeGrafico
        {
            get
            {
                return this._opcoesGraficoTracao;
            }
            set
            {
                this._opcoesGraficoTracao = value;
            }
        }
        [DataMember]
        public List<InformacaoAdicional> InformacoesAdicionais
        {
            get
            {
                return this._informacoesAdicionais;
            }
            set
            {
                this._informacoesAdicionais = value;
            }
        }

        private ModeloDeEnsaioTracao()
        {
            this.CorpoDeProva = new Quadrado(2.5);
            this.OpcoesDeGrafico = new OpcoesGraficoTracao();
            this.InformacoesAdicionais = new List<InformacaoAdicional>(){
                new InformacaoAdicional("Descrição de ensaio", ""),
                new InformacaoAdicional("Empresa", ""),
                new InformacaoAdicional("Operador", ""),
                new InformacaoAdicional("Cliente", ""),
                new InformacaoAdicional("Material", ""),
                new InformacaoAdicional("Lote", ""),
                new InformacaoAdicional("Nº do pedido", ""),
                new InformacaoAdicional("Nº da amostra", ""),
                new InformacaoAdicional("Nº do certificado", ""),
                new InformacaoAdicional("Resultado", ""),
                new InformacaoAdicional("Observação", ""),
            };
        }

        public static ModeloDeEnsaioTracao getInstance()
        {
            if (_instance == null)
            {
                _instance = new ModeloDeEnsaioTracao();
                _instance.abrirModeloDeEnsaio();
            }
            return _instance;
        }

        #endregion

        #region Ensaio

        public Ensaio criaNovoEnsaio()
        {
            EnsaioTracao novoEnsaio = new EnsaioTracao();
            novoEnsaio.Grafico.OpcoesGrafico = (OpcoesGraficoTracao)this.OpcoesDeGrafico.Clone();
            novoEnsaio.CorpoDeProva = (CorpoDeProva)this.CorpoDeProva.Clone();
            novoEnsaio.InformacoesAdicionais = this.InformacoesAdicionais.Select(i => (InformacaoAdicional)i.Clone()).ToList();
            return novoEnsaio;
        }

        public void usarEnsaioComoModelo(Ensaio ensaio)
        {
            EnsaioTracao ensaioTracao = (EnsaioTracao)ensaio;
            this.CorpoDeProva = (CorpoDeProva)ensaioTracao.CorpoDeProva.Clone();
            this.InformacoesAdicionais = ensaioTracao.InformacoesAdicionais.Select(i => (InformacaoAdicional)i.Clone()).ToList();
            this.OpcoesDeGrafico = (OpcoesGraficoTracao)ensaioTracao.Grafico.OpcoesGrafico.Clone();

            Logging.getInstance().info(Properties.Resources.usarEnsaioComoModeloString, true);
            this.salvarModeloDeEnsaio();
        }

        #endregion

        #region Abrir/Salvar

        public void salvarModeloDeEnsaio()
        {
            this.salvarModeloDeEnsaio(NOME_ARQUIVO_MODELO);
        }

        public void salvarModeloDeEnsaio(String nomeDoArquivo)
        {
            Stream stream = new FileStream(nomeDoArquivo, FileMode.Create);
            Serializador.serializaObjeto(stream, this, typeof(ModeloDeEnsaioTracao));
            stream.Close();
        }

        public void abrirModeloDeEnsaio()
        {
            if (File.Exists(NOME_ARQUIVO_MODELO))
            {
                this.abrirModeloDeEnsaio(NOME_ARQUIVO_MODELO);
            }
        }

        public void abrirModeloDeEnsaio(String nomeDoArquivo)
        {
            try
            {
                Stream stream = new FileStream(nomeDoArquivo, FileMode.Open);
                ModeloDeEnsaioTracao modelo = (ModeloDeEnsaioTracao)Serializador.deserializaObjeto(stream, typeof(ModeloDeEnsaioTracao));
                this.abrirModeloDeEnsaio(modelo);
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(Properties.Resources.impossivelCarregarModeloString, ex, true);
            }
        }

        public void abrirModeloDeEnsaio(ModeloDeEnsaioTracao modelo)
        {
            this.CorpoDeProva = modelo.CorpoDeProva;
            this.OpcoesDeGrafico = modelo.OpcoesDeGrafico;
            this.InformacoesAdicionais = modelo.InformacoesAdicionais;
        }

        #endregion
    }
}
