﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.MyMath;
using System.Runtime.Serialization;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    public class OpcaoModuloElasticidadeAjustado : ICloneable
    {
        #region Construtor e membros

        private ReferenciaAjusteModuloElasticidade _referencia;

        [DataMember]
        public ReferenciaAjusteModuloElasticidade Referencia
        {
            get
            {
                return this._referencia;
            }
            set
            {
                this._referencia = value;
                this.informReferenciaListeners();
            }
        }

        public OpcaoModuloElasticidadeAjustado()
        {
            this.Referencia = new PercentualTensaoEscoamento();
        }

        #endregion

        #region Misc.

        public object Clone()
        {
            OpcaoModuloElasticidadeAjustado opcao = new OpcaoModuloElasticidadeAjustado();
            opcao.Referencia = (ReferenciaAjusteModuloElasticidade)this.Referencia.Clone();
            return opcao;
        }

        #endregion

        #region Eventos

        #region ReferenciaChanged

        public delegate void ReferenciaChangedEvent(object sender, ReferenciaChangedEventArgs e);

        public event ReferenciaChangedEvent ReferenciaChanged;

        public class ReferenciaChangedEventArgs : EventArgs
        {
            public ReferenciaChangedEventArgs(ReferenciaAjusteModuloElasticidade referencia)
            {
                this.Referencia = referencia;
            }

            public ReferenciaAjusteModuloElasticidade Referencia
            {
                get;
                set;
            }
        }

        private void informReferenciaListeners()
        {
            if (ReferenciaChanged != null)
            {
                ReferenciaChanged(this, new ReferenciaChangedEventArgs(this.Referencia));
            }
        }

        #endregion

        #endregion
    }

    [DataContract]
    [KnownType(typeof(PercentualFundoEscala))]
    [KnownType(typeof(PercentualTensaoEscoamento))]
    [KnownType(typeof(TensaoAbsoluta))]
    public abstract class ReferenciaAjusteModuloElasticidade : ICloneable
    {
        #region Membros

        private double _pontoInicial;
        private double _pontoFinal;

        [DataMember]
        public Double PontoInferior
        {
            get
            {
                return this._pontoInicial;
            }
            set
            {
                this._pontoInicial = value;
            }
        }
        [DataMember]
        public Double PontoSuperior
        {
            get
            {
                return this._pontoFinal;
            }
            set
            {
                this._pontoFinal = value;
            }
        }

        #endregion

        #region Misc.

        public abstract object Clone();

        #endregion
    }

    public class PercentualFundoEscala : ReferenciaAjusteModuloElasticidade
    {
        #region Construtor e membros

        public PercentualFundoEscala()
        {
            this.PontoSuperior = 0.4;
            this.PontoInferior = 0.2;
        }

        #endregion

        #region Misc

        public override object Clone()
        {
            PercentualFundoEscala referencia = new PercentualFundoEscala();
            referencia.PontoSuperior = this.PontoSuperior;
            referencia.PontoInferior = this.PontoInferior;
            return referencia;
        }

        public override string ToString()
        {
            return String.Format("{0} ({1})", Properties.Resources.percentualFundoEscalaString, Properties.Resources.percentualFundoEscalaUnidadeString);
        }

        #endregion
    }

    public class PercentualTensaoEscoamento : ReferenciaAjusteModuloElasticidade
    {
        #region Construtor e membros

        public PercentualTensaoEscoamento()
        {
            this.PontoSuperior = 0.4;
            this.PontoInferior = 0.2;
        }

        #endregion

        #region Misc

        public override object Clone()
        {
            PercentualTensaoEscoamento referencia = new PercentualTensaoEscoamento();
            referencia.PontoSuperior = this.PontoSuperior;
            referencia.PontoInferior = this.PontoInferior;
            return referencia;
        }

        public override string ToString()
        {
            return String.Format("{0} ({1})", Properties.Resources.percentualTensaoEscoamentoString, Properties.Resources.percentualTensaoEscoamentoUnidadeString);
        }

        #endregion
    }

    public class TensaoAbsoluta : ReferenciaAjusteModuloElasticidade
    {
        #region Construtor e membros

        private Unidade _unidade;

        [DataMember]
        public Unidade UnidadeTensao
        {
            get
            {
                return this._unidade;
            }
            set
            {
                this._unidade = value;
            }
        }

        public TensaoAbsoluta()
        {
            this.PontoSuperior = 500;
            this.PontoInferior = 200;
            this.UnidadeTensao = Unidade.MPA;
        }

        #endregion

        #region Misc

        public override object Clone()
        {
            TensaoAbsoluta referencia = new TensaoAbsoluta();
            referencia.PontoSuperior = this.PontoSuperior;
            referencia.PontoInferior = this.PontoInferior;
            referencia.UnidadeTensao = this.UnidadeTensao;
            return referencia;
        }

        public override string ToString()
        {
            return String.Format("{0}", Properties.Resources.tensaoAbsolutaString);
        }

        #endregion
    }
}
