﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    public class OpcoesEnsaioTracao : ICloneable
    {
        #region Construtor e membros

        private bool _detectarRompimento;
        private double _limiarDeRompimento;
        private OpcaoModuloElasticidadeTempoReal _opcaoModuloTempoReal;
        private OpcoesRemocaoExtensometro _referenciaRemocaoExtensometro;
        private OpcaoModuloElasticidadeAjustado _opcaoModuloAjustado;
        private double _distanciaParalelaExtensometro;
        private double _distanciaParalelaEncoder;

        [DataMember]
        public Boolean DetectarRompimento
        {
            get
            {
                return this._detectarRompimento;
            }
            set
            {
                this._detectarRompimento = value;
            }
        }
        [DataMember]
        public double LimiarDeRompimento
        {
            get
            {
                return this._limiarDeRompimento;
            }
            set
            {
                this._limiarDeRompimento = value;
            }
        }
        [DataMember]
        public OpcaoModuloElasticidadeTempoReal OpcaoModuloTempoReal
        {
            get
            {
                return this._opcaoModuloTempoReal;
            }
            set
            {
                this._opcaoModuloTempoReal = value;
            }
        }
        [DataMember]
        public OpcaoModuloElasticidadeAjustado OpcaoModuloAjustado
        {
            get
            {
                return this._opcaoModuloAjustado;
            }
            set
            {
                this._opcaoModuloAjustado = value;
            }
        }
        [DataMember]
        public OpcoesRemocaoExtensometro OpcaoRemocaoExtensometro
        {
            get
            {
                return this._referenciaRemocaoExtensometro;
            }
            set
            {
                this._referenciaRemocaoExtensometro = value;
            }
        }
        [DataMember]
        public double DistanciaParalelaExtensometro
        {
            get
            {
                return this._distanciaParalelaExtensometro;
            }
            set
            {
                this._distanciaParalelaExtensometro = value;
            }
        }
        [DataMember]
        public double DistanciaParalelaEncoder
        {
            get
            {
                return this._distanciaParalelaEncoder;
            }
            set
            {
                this._distanciaParalelaEncoder = value;
            }
        }

        public OpcoesEnsaioTracao()
        {
            this.DetectarRompimento = true;
            this.LimiarDeRompimento = 0.75;
            this.OpcaoModuloTempoReal = new OpcaoModuloElasticidadeTempoReal();
            this.OpcaoRemocaoExtensometro = new RemocaoExtensometroParalela();
            this.OpcaoModuloAjustado = new OpcaoModuloElasticidadeAjustado();
            this.DistanciaParalelaExtensometro = 0.2;
            this.DistanciaParalelaEncoder = 0.2;
        }

        #endregion

        #region Salvar opcoes

        public void salvaOpcoesEnsaio(OpcoesEnsaioTracao opcoes)
        {
            this.DetectarRompimento = opcoes.DetectarRompimento;
            this.LimiarDeRompimento = opcoes.LimiarDeRompimento;
            this.DistanciaParalelaExtensometro = opcoes.DistanciaParalelaExtensometro;
            this.DistanciaParalelaEncoder = opcoes.DistanciaParalelaEncoder;
            this.OpcaoRemocaoExtensometro = (OpcoesRemocaoExtensometro)opcoes.OpcaoRemocaoExtensometro.Clone();
            this.OpcaoModuloTempoReal = (OpcaoModuloElasticidadeTempoReal)opcoes.OpcaoModuloTempoReal.Clone();
            this.OpcaoModuloAjustado = (OpcaoModuloElasticidadeAjustado)opcoes.OpcaoModuloAjustado.Clone();
            this.informOpcoesDeEnsaioListeners();
        }

        #endregion

        #region Eventos

        #region OpcoesDeEnsaioChanged

        public delegate void OpcoesDeEnsaioChangedEvent(object sender, OpcoesDeEnsaioChangedEventArgs e);

        public event OpcoesDeEnsaioChangedEvent OpcoesDeEnsaioChanged;

        public class OpcoesDeEnsaioChangedEventArgs : EventArgs
        {
            public OpcoesDeEnsaioChangedEventArgs(OpcoesEnsaioTracao opcoes)
            {
                this.OpcoesEnsaioTracao = opcoes;
            }

            public OpcoesEnsaioTracao OpcoesEnsaioTracao
            {
                get;
                set;
            }
        }

        private void informOpcoesDeEnsaioListeners()
        {
            if (OpcoesDeEnsaioChanged != null)
            {
                OpcoesDeEnsaioChanged(this, new OpcoesDeEnsaioChangedEventArgs((OpcoesEnsaioTracao)this.Clone()));
            }
        }

        #endregion

        #endregion

        #region Misc

        public object Clone()
        {
            OpcoesEnsaioTracao opcoes = new OpcoesEnsaioTracao();
            opcoes.DetectarRompimento = this.DetectarRompimento;
            opcoes.LimiarDeRompimento = this.LimiarDeRompimento;
            opcoes.DistanciaParalelaEncoder = this.DistanciaParalelaEncoder;
            opcoes.DistanciaParalelaExtensometro = this.DistanciaParalelaExtensometro;
            opcoes.OpcaoRemocaoExtensometro = (OpcoesRemocaoExtensometro)this.OpcaoRemocaoExtensometro.Clone();
            opcoes.OpcaoModuloTempoReal = (OpcaoModuloElasticidadeTempoReal)this.OpcaoModuloTempoReal.Clone();
            opcoes.OpcaoModuloAjustado = (OpcaoModuloElasticidadeAjustado)this.OpcaoModuloAjustado.Clone();
            return opcoes;
        }

        #endregion
    }
}
