﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using TestView.MyMath;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    [KnownType(typeof(RemocaoExtensometroParalela))]
    [KnownType(typeof(RemocaoExtensometroPosicao))]
    [KnownType(typeof(RemocaoExtensometroDeformacao))]
    public abstract class OpcoesRemocaoExtensometro : ICloneable
    {
        [DataMember]
        public abstract double Limiar
        {
            get;
            set;
        }
        public abstract Unidade UnidadeLimiar
        {
            get;
        }

        public abstract object Clone();
    }

    public class RemocaoExtensometroParalela : OpcoesRemocaoExtensometro
    {
        #region Construtor e membros

        private static string descricao = "Paralela";
        private static Unidade unidade = Unidade.PERCENTUAL_EXT;
        private double _limiar;

        public override double Limiar
        {
            get
            {
                return this._limiar;
            }
            set
            {
                this._limiar = value;
            }
        }
        public override Unidade UnidadeLimiar
        {
            get
            {
                return unidade;
            }
        }

        public RemocaoExtensometroParalela()
        {
            this._limiar = 0.8;
        }

        public RemocaoExtensometroParalela(double limiar)
        {
            this._limiar = limiar;
        }

        public override string  ToString()
        {
            return descricao;
        }

        public override object Clone()
        {
            RemocaoExtensometroParalela clone = new RemocaoExtensometroParalela();
            clone.Limiar = this.Limiar;
            return clone;
        }

        #endregion
    }

    public class RemocaoExtensometroPosicao : OpcoesRemocaoExtensometro
    {
        #region Construtor e membros

        private static string descricao = "Posição";
        private static Unidade unidade = Unidade.MM;
        private double _limiar;

        public override double Limiar
        {
            get
            {
                return this._limiar;
            }
            set
            {
                this._limiar = value;
            }
        }
        public override Unidade UnidadeLimiar
        {
            get
            {
                return unidade;
            }
        }

        public RemocaoExtensometroPosicao()
        {
            this._limiar = 1.0;
        }

        public RemocaoExtensometroPosicao(double limiar)
        {
            this._limiar = limiar;
        }

        public override string ToString()
        {
            return descricao;
        }

        public override object Clone()
        {
            RemocaoExtensometroPosicao clone = new RemocaoExtensometroPosicao();
            clone.Limiar = this.Limiar;
            return clone;
        }

        #endregion
    }

    public class RemocaoExtensometroDeformacao : OpcoesRemocaoExtensometro
    {
        #region Construtor e membros

        private static string descricao = "Deformação";
        private static Unidade unidade = Unidade.PERCENTUAL_EXT;
        private double _limiar;

        public override double Limiar
        {
            get
            {
                return this._limiar;
            }
            set
            {
                this._limiar = value;
            }
        }
        public override Unidade UnidadeLimiar
        {
            get
            {
                return unidade;
            }
        }

        public RemocaoExtensometroDeformacao()
        {
            this._limiar = 0.01;
        }

        public RemocaoExtensometroDeformacao(double limiar)
        {
            this._limiar = limiar;
        }

        public override string ToString()
        {
            return descricao;
        }

        public override object Clone()
        {
            RemocaoExtensometroDeformacao clone = new RemocaoExtensometroDeformacao();
            clone.Limiar = this.Limiar;
            return clone;
        }

        #endregion
    }
}
