﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Ensaios.Graficos;
using TestView.Ensaios;
using TestView.MyMath;
using ModuloEnsaioTracao.Ensaios;
using ModuloEnsaioTracao.Graficos;

namespace ModuloEnsaioTracao.GUI
{
    public partial class FormOpcoesDeGrafico : Form
    {
        #region Construtor e membros

        List<TipoCoordenada> _tiposAbscissa = new List<TipoCoordenada>(){
                TipoCoordenada.TEMPO,
                TipoCoordenada.DEFORMACAO
            };

        List<TipoCoordenada> _tiposOrdenada = new List<TipoCoordenada>(){
                TipoCoordenada.FORCA,
                TipoCoordenada.TENSAO
            };

        private ModuloEnsaioTracao _moduloEnsaio;
        private OpcoesGraficoTracao _opcoes;

        private EnsaioTracao EnsaioTracao
        {
            get
            {
                return (EnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio.Ensaio;
            }
        }
        private RotinaDeEnsaioTracao RotinaEnsaioTracao
        {
            get
            {
                return (RotinaDeEnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio;
            }
        }

        public FormOpcoesDeGrafico(IModuloDeEnsaio moduloEnsaio)
        {
            InitializeComponent();
            this._moduloEnsaio = (ModuloEnsaioTracao)moduloEnsaio;
            this._opcoes = (OpcoesGraficoTracao)this.EnsaioTracao.Grafico.OpcoesGrafico.Clone();
            preencheComboboxes();
            selecionaComboboxes();
            selecionaCheckboxes();
        }

        #endregion

        #region Componentes do Form

        private void preencheComboboxes()
        {
            preencheTipoAbscissa();
            preencheTipoOrdenada();
        }

        private void preencheTipoAbscissa()
        {
            this.comboBoxTipoAbscissa.Items.Clear();
            foreach (TipoCoordenada tipoCoordenada in _tiposAbscissa)
            {
                this.comboBoxTipoAbscissa.Items.Add(tipoCoordenada);
            }
        }

        private void preencheTipoOrdenada()
        {
            this.comboBoxTipoOrdenada.Items.Clear();
            foreach (TipoCoordenada tipoCoordenada in _tiposOrdenada)
            {
                this.comboBoxTipoOrdenada.Items.Add(tipoCoordenada);
            }
        }

        private void preencheUnidadeAbscissa()
        {
            this.comboBoxUnidadeAbscissa.Items.Clear();
            TipoCoordenada tipoAbscissa = (TipoCoordenada) this.comboBoxTipoAbscissa.SelectedItem;
            if (tipoAbscissa.Equals(TipoCoordenada.DEFORMACAO))
            {
                this.comboBoxUnidadeAbscissa.Items.AddRange(Unidade.listaUnidadesDeDeformacao());
            }
            else if (tipoAbscissa.Equals(TipoCoordenada.TEMPO))
            {
                this.comboBoxUnidadeAbscissa.Items.AddRange(Unidade.listaUnidadesDeTempo());
            }
        }

        private void preencheUnidadeOrdenada()
        {
            this.comboBoxUnidadeOrdenada.Items.Clear();
            TipoCoordenada tipoOrdenada = (TipoCoordenada)this.comboBoxTipoOrdenada.SelectedItem;
            if (tipoOrdenada.Equals(TipoCoordenada.FORCA))
            {
                this.comboBoxUnidadeOrdenada.Items.AddRange(Unidade.listaUnidadesDeForca());
            }
            else if (tipoOrdenada.Equals(TipoCoordenada.TENSAO))
            {
                this.comboBoxUnidadeOrdenada.Items.AddRange(Unidade.listaUnidadesDeTensao());
            }
        }

        private void selecionaComboboxes()
        {
            selecionaComboboxTipoAbscissa();
            selecionaComboboxTipoOrdenada();
        }

        private void selecionaComboboxTipoAbscissa()
        {
            TipoCoordenada tipoAbscissa = this._opcoes.TipoAbscissa;
            this.comboBoxTipoAbscissa.SelectedItem = tipoAbscissa;
        }

        private void selecionaComboboxTipoOrdenada()
        {
            TipoCoordenada tipoOrdenada = this._opcoes.TipoOrdenada;
            this.comboBoxTipoOrdenada.SelectedItem = tipoOrdenada;
        }

        private void selecionaComboboxUnidadeAbscissa()
        {
            Unidade unidadeAbscissa = this._opcoes.UnidadeAbscissa;
            if (this.comboBoxUnidadeAbscissa.Items.Contains(unidadeAbscissa))
            {
                this.comboBoxUnidadeAbscissa.SelectedItem = unidadeAbscissa;
            }
            else if (this.comboBoxUnidadeAbscissa.Items.Count > 0)
            {
                this.comboBoxUnidadeAbscissa.SelectedIndex = 0;
            }
        }

        private void selecionaComboboxUnidadeOrdenada()
        {
            Unidade unidadeOrdenada = this._opcoes.UnidadeOrdenada;
            if (this.comboBoxUnidadeOrdenada.Items.Contains(unidadeOrdenada))
            {
                this.comboBoxUnidadeOrdenada.SelectedItem = unidadeOrdenada;
            }
            else if (this.comboBoxUnidadeOrdenada.Items.Count > 0)
            {
                this.comboBoxUnidadeOrdenada.SelectedIndex = 0;
            }
        }

        private void selecionaCheckboxes()
        {
            this.checkBoxExibirModulo.Checked = this._opcoes.ExibirModuloElasticidade;
            this.checkBoxExibirParalelaModulo.Checked = this._opcoes.ExibirParalelaModulo;
        }

        #endregion

        #region Opções do gráfico

        private void salvaOpcoes()
        {
            this.RotinaEnsaioTracao.salvaOpcoesGrafico(this._opcoes);
        }

        #endregion

        #region Eventos

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonAplicar_Click(object sender, EventArgs e)
        {
            salvaOpcoes();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            salvaOpcoes();
            this.Close();
        }

        private void comboBoxTipoOrdenada_SelectedIndexChanged(object sender, EventArgs e)
        {
            this._opcoes.TipoOrdenada = (TipoCoordenada)this.comboBoxTipoOrdenada.SelectedItem;
            this.preencheUnidadeOrdenada();
            this.selecionaComboboxUnidadeOrdenada();
        }

        private void comboBoxTipoAbscissa_SelectedIndexChanged(object sender, EventArgs e)
        {
            this._opcoes.TipoAbscissa = (TipoCoordenada)this.comboBoxTipoAbscissa.SelectedItem;
            this.preencheUnidadeAbscissa();
            this.selecionaComboboxUnidadeAbscissa();
        }

        private void comboBoxUnidadeOrdenada_SelectedIndexChanged(object sender, EventArgs e)
        {
            this._opcoes.UnidadeOrdenada = (Unidade)this.comboBoxUnidadeOrdenada.SelectedItem;
        }

        private void comboBoxUnidadeAbscissa_SelectedIndexChanged(object sender, EventArgs e)
        {
            this._opcoes.UnidadeAbscissa = (Unidade)this.comboBoxUnidadeAbscissa.SelectedItem;
        }

        private void checkBoxExibirModulo_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoes.ExibirModuloElasticidade = this.checkBoxExibirModulo.Checked;
        }

        private void checkBoxExibirParalelaModulo_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoes.ExibirParalelaModulo = this.checkBoxExibirParalelaModulo.Checked;
        }

        #endregion
    }
}
