﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ModuloEnsaioTracao.Ensaios;
using System.IO;
using TestView.Seguranca;
using TestView.Ensaios;

namespace ModuloEnsaioTracao.GUI
{
    public partial class FormOpcoesDoRelatorio : Form
    {
        #region Construtor e membros

        private ModuloEnsaioTracao _moduloEnsaio;
        private OpcoesRelatorio _opcoesRelatorio;

        private EnsaioTracao EnsaioTracao {
            get
            {
                return (EnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio.Ensaio;
            }
        }
        private RotinaDeEnsaioTracao RotinaEnsaioTracao
        {
            get
            {
                return (RotinaDeEnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio;
            }
        }

        public FormOpcoesDoRelatorio(IModuloDeEnsaio moduloEnsaio)
        {
            InitializeComponent();
            this._moduloEnsaio = (ModuloEnsaioTracao)moduloEnsaio;
            this._opcoesRelatorio = (OpcoesRelatorio)this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.Clone();
            this._opcoesRelatorio.CaminhoLogotipoChanged += new OpcoesRelatorio.CaminhoLogotipoChangedEvent(caminhoLogotipoChanged);
            this.inicializaLayout();
        }

        #endregion

        #region Layout

        private void inicializaLayout()
        {
            this.preencheCheckboxes();
            this.preencheTextboxes();
            this.exibeImagemLogotipo();
        }

        private void preencheCheckboxes()
        {
            this.checkBoxInformacoesAdicionais.Checked = this._opcoesRelatorio.InformacoesAdicionais;
            this.checkBoxForcaMaxima.Checked = this._opcoesRelatorio.ForcaMaxima;
            this.checkBoxTensaoMaxima.Checked = this._opcoesRelatorio.TensaoMaxima;
            this.checkBoxModuloElasticidade.Checked = this._opcoesRelatorio.ModuloElasticidade;
            this.checkBoxForcaEscoamento.Checked = this._opcoesRelatorio.ForcaEscoamento;
            this.checkBoxTensaoEscoamento.Checked = this._opcoesRelatorio.TensaoEscoamento;
            this.checkBoxArea.Checked = this._opcoesRelatorio.AreaInicial;
            this.checkBoxIncluirLogotipo.Checked = this._opcoesRelatorio.Logotipo;
        }

        #region Textboxes

        private void preencheTextboxes()
        {
            this.preencheTextboxInformacoesEmpresa();
            this.preencheTextboxCaminhoLogo();
        }

        private void preencheTextboxInformacoesEmpresa()
        {
            this.preencheTextboxInformacoesEmpresa(this._opcoesRelatorio.InformacoesEmpresa);
        }

        private void preencheTextboxInformacoesEmpresa(String informacoes)
        {
            this.textBoxInformacoesEmpresa.Text = informacoes;
        }

        private void preencheTextboxCaminhoLogo()
        {
            this.preencheTextboxCaminhoLogo(this._opcoesRelatorio.CaminhoLogotipo);
        }

        private void preencheTextboxCaminhoLogo(String caminho)
        {
            this.textBoxCaminhoLogotipo.Text = caminho;
        }

        #endregion

        private void exibeImagemLogotipo()
        {
            this.exibeImagemLogotipo(this._opcoesRelatorio.CaminhoLogotipo);
        }

        private void exibeImagemLogotipo(String caminhoImagemLogo)
        {
            try
            {
                if (caminhoImagemLogo != null && File.Exists(caminhoImagemLogo))
                {
                    Image imagemLogo = Image.FromFile(caminhoImagemLogo);
                    this.pictureBoxLogotipo.Image = imagemLogo;
                }
            }
            catch (Exception ex)
            {
                this.pictureBoxLogotipo.Image = null;
                Logging.getInstance().warn(Properties.Resources.impossivelExibirLogotipoString, ex, true);
            }
        }

        #endregion

        #region Opcoes

        private void salvaOpcoesRelatorio()
        {
            this.RotinaEnsaioTracao.salvaOpcoesRelatorio(this._opcoesRelatorio);
        }

        #endregion

        #region Eventos

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonAplicar_Click(object sender, EventArgs e)
        {
            this.salvaOpcoesRelatorio();
        }

        private void checkBoxInformacoesAdicionais_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.InformacoesAdicionais = this.checkBoxInformacoesAdicionais.Checked;
        }

        private void checkBoxForcaMaxima_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.ForcaMaxima = this.checkBoxForcaMaxima.Checked;
        }

        private void checkBoxTensaoMaxima_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.TensaoMaxima = this.checkBoxTensaoMaxima.Checked;
        }

        private void checkBoxModuloElasticidade_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.ModuloElasticidade = this.checkBoxModuloElasticidade.Checked;
        }

        private void checkBoxArea_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.AreaInicial = this.checkBoxArea.Checked;
        }

        private void checkBoxTensaoEscoamento_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.TensaoEscoamento = this.checkBoxTensaoEscoamento.Checked;
        }

        private void checkBoxForcaEscoamento_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.ForcaEscoamento = this.checkBoxForcaEscoamento.Checked;
        }

        private void textBoxInformacoesEmpresa_TextChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.InformacoesEmpresa = this.textBoxInformacoesEmpresa.Text;
        }

        private void buttonProcurarLogotipo_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialogoLogotipo = new OpenFileDialog();
            DialogResult resultado = dialogoLogotipo.ShowDialog();
            if (resultado == DialogResult.OK)
            {
                String nomeArquivo = dialogoLogotipo.FileName;
                this._opcoesRelatorio.CaminhoLogotipo = nomeArquivo;
            }
        }

        private void caminhoLogotipoChanged(object sender, OpcoesRelatorio.CaminhoLogotipoChangedEventArgs e)
        {
            this.preencheTextboxCaminhoLogo(e.CaminhoLogotipo);
            this.exibeImagemLogotipo(e.CaminhoLogotipo);
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            this.salvaOpcoesRelatorio();
            this.Close();
        }

        private void checkBoxIncluirLogotipo_CheckedChanged(object sender, EventArgs e)
        {
            this._opcoesRelatorio.Logotipo = this.checkBoxIncluirLogotipo.Checked;
        }

        #endregion
    }
}
