﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ModuloEnsaioTracao.Ensaios;
using TestView.Ensaios.Utilidades;
using System.IO;
using TestView.MyMath;
using TestView.Ensaios;

namespace ModuloEnsaioTracao.GUI
{
    public partial class FormRelatorioEnsaioTracao : Form
    {
        #region Construtor e membros

        private ModuloEnsaioTracao _moduloEnsaio;
        private EnsaioTracao EnsaioTracao
        {
            get
            {
                return (EnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio.Ensaio;
            }
        }
        private ModuloTracaoGUI ModuloTracaoGUI
        {
            get
            {
                return (ModuloTracaoGUI)this._moduloEnsaio.FachadaGUI;
            }
        }

        public FormRelatorioEnsaioTracao(IModuloDeEnsaio moduloEnsaio)
        {
            InitializeComponent();
            this._moduloEnsaio = (ModuloEnsaioTracao)moduloEnsaio;
            this.inicializaLayout();
        }

        #endregion

        #region Layout

        private void inicializaLayout()
        {
            this.ininicializaLabelDataEnsaio();
            this.inicializaLabelInformacoesEmpresa();
            this.inicializaLabelInformacoesAdicionais();
            this.inicializaLabelDadosEnsaio();
            this.inicializaImagemLogotipo();
            this.inicializaImagemGrafico();
        }

        #region Labels

        private void ininicializaLabelDataEnsaio()
        {
            this.labelDataEnsaio.Text = String.Format("{0}: {1}", Properties.Resources.dataEnsaioString, this.EnsaioTracao.DataEnsaio);
        }

        private void inicializaLabelInformacoesEmpresa()
        {
            this.labelInformacoesEmpresa.Text = this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.InformacoesEmpresa;
        }

        private void inicializaLabelInformacoesAdicionais()
        {
            String informacoesAdicionais = String.Empty;
            if (this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.InformacoesAdicionais)
            {
                for (int i = 0; i < this.EnsaioTracao.InformacoesAdicionais.Count; i++)
                {
                    InformacaoAdicional informacao = this.EnsaioTracao.InformacoesAdicionais[i];
                    String stringInformacao = String.Format("{0}: {1}", informacao.Informacao, informacao.Valor);
                    informacoesAdicionais += stringInformacao + Environment.NewLine;
                }
            }
            this.labelInformacoesAdicionais.Text = informacoesAdicionais;
        }

        private void inicializaLabelDadosEnsaio()
        {
            String dadosEnsaio = String.Empty;
            OpcoesRelatorio opcoes = this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio;
            #region Força máxima
            if (opcoes.ForcaMaxima)
            {
                String forcaMaximaString = String.Format("{0}: {1}", Properties.Resources.forcaMaximaString, Properties.Resources.indicacaoVaziaString);
                Double forcaMaxima = this.EnsaioTracao.ForcaMaxima;
                if (!double.IsNaN(forcaMaxima))
                {
                    Unidade unidadeForcaMaxima = this.EnsaioTracao.UnidadeForcaMaxima;
                    forcaMaximaString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.forcaMaximaString, forcaMaxima, unidadeForcaMaxima.ToString());
                }
                dadosEnsaio += Environment.NewLine + forcaMaximaString;
            }
            #endregion
            #region Tensão máxima
            if (opcoes.TensaoMaxima)
            {
                String tensaoMaximaString = String.Format("{0}: {1}", Properties.Resources.tensaoMaximaString, Properties.Resources.indicacaoVaziaString);
                Double tensaoMaxima = this.EnsaioTracao.TensaoMaxima;
                if (!double.IsNaN(tensaoMaxima))
                {
                    Unidade unidadeTensaoMaxima = this.EnsaioTracao.UnidadeTensaoMaxima;
                    tensaoMaximaString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.tensaoMaximaString, tensaoMaxima, unidadeTensaoMaxima.ToString());
                }
                dadosEnsaio += Environment.NewLine + tensaoMaximaString;
            }
            #endregion
            #region Módulo de elasticidade
            if (opcoes.ModuloElasticidade)
            {
                String moduloElasticidadeString = String.Format("{0}: {1}", Properties.Resources.moduloElasticidadeString, Properties.Resources.indicacaoVaziaString);
                Double modulo = this.EnsaioTracao.ModuloDeElasticidade;
                if (!double.IsNaN(modulo))
                {
                    Unidade unidadeModulo = this.EnsaioTracao.UnidadeModuloElasticidade;
                    Double moduloGPa = Conversor.converterTensao(modulo, unidadeModulo, Unidade.GPA);
                    moduloElasticidadeString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.moduloElasticidadeString, moduloGPa, Unidade.GPA.ToString());
                }
                dadosEnsaio += Environment.NewLine + moduloElasticidadeString;
            }
            #endregion
            #region Tensão escoamento
            if (opcoes.TensaoEscoamento)
            {
                String tensaoEscoamentoString = String.Format("{0}: {1}", Properties.Resources.tensaoEscoamentoString, Properties.Resources.indicacaoVaziaString);
                Double tensaoEscoamento = this.EnsaioTracao.TensaoEscoamento;
                if (!double.IsNaN(tensaoEscoamento))
                {
                    Unidade unidadeTensaoEscoamento = this.EnsaioTracao.UnidadeTensaoEscoamento;
                    tensaoEscoamentoString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.tensaoEscoamentoString, tensaoEscoamento, unidadeTensaoEscoamento.ToString());
                }
                dadosEnsaio += Environment.NewLine + tensaoEscoamentoString;
            }
            #endregion
            #region Força escoamento
            if (opcoes.ForcaEscoamento)
            {
                String forcaEscoamentoString = String.Format("{0}: {1}", Properties.Resources.forcaEscoamentoString, Properties.Resources.indicacaoVaziaString);
                Double forcaEscoamento = this.EnsaioTracao.ForcaEscoamento;
                if (!double.IsNaN(forcaEscoamento))
                {
                    Unidade unidadeForcaEscoamento = this.EnsaioTracao.UnidadeForcaEscoamento;
                    forcaEscoamentoString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.forcaEscoamentoString, forcaEscoamento, unidadeForcaEscoamento.ToString());
                }
                dadosEnsaio += Environment.NewLine + forcaEscoamentoString;
            }
            #endregion
            #region Área Inicial
            if (opcoes.AreaInicial)
            {
                String areaInicialString = String.Format("{0}: {1}", Properties.Resources.areaInicialString, Properties.Resources.indicacaoVaziaString);
                Double areaInicial = this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                if (!double.IsNaN(areaInicial))
                {
                    Unidade unidadeArea = Unidade.MM2;
                    areaInicialString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.areaInicialString, areaInicial, unidadeArea.ToString());
                }
                dadosEnsaio += Environment.NewLine + areaInicialString;
            }
            #endregion
            this.labelDadosEnsaio.Text = dadosEnsaio;
        }

        #endregion

        #region Pictureboxes

        private void inicializaImagemLogotipo()
        {
            Boolean exibeImagem = this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.Logotipo;
            String caminhoImagemLogo = this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.CaminhoLogotipo;
            if (exibeImagem && caminhoImagemLogo != null && File.Exists(caminhoImagemLogo))
            {
                try
                {
                    {
                        Image imagemLogo = Image.FromFile(caminhoImagemLogo);
                        this.pictureBoxLogoEmpresa.Image = imagemLogo;
                    }
                }
                catch (Exception)
                {
                    this.pictureBoxLogoEmpresa.Image = null;
                }
            }
            else
            {
                this.pictureBoxLogoEmpresa.Image = null;
            }
        }

        private void inicializaImagemGrafico()
        {
            this.pictureBoxGrafico.Image = this.ModuloTracaoGUI.ImagemGrafico;
        }

        #endregion

        #endregion
    }
}
