﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Ensaios.Graficos;
using System.Runtime.Serialization;

namespace ModuloEnsaioTracao.Graficos
{
    [DataContract]
    public class GraficoEnsaioTracao
    {
        #region Construtor e membros

        private List<PontoGrafico> _seriePrincipal;
        private List<PontoGrafico> _serieModuloElasticidade;
        private List<PontoGrafico> _serieParalelaModulo;
        private OpcoesGraficoTracao _opcoesGrafico;
        private List<Double> _seriePrincipalValoresOrdenada;
        private List<Double> _seriePrincipalValoresAbscissa;
        private List<Double> _serieModuloElasticidadeValoresOrdenada;
        private List<Double> _serieModuloElasticidadeValoresAbscissa;
        private List<Double> _serieParalelaModuloValoresOrdenada;
        private List<Double> _serieParalelaModuloValoresAbscissa;

        public String TituloOrdenada
        {
            get
            {
                return String.Format("{0} ({1})", this.OpcoesGrafico.TipoOrdenada.ToString(), this.OpcoesGrafico.UnidadeOrdenada.ToString());
            }
        }
        public String TituloAbscissa
        {
            get
            {
                return String.Format("{0} ({1})", this.OpcoesGrafico.TipoAbscissa.ToString(), this.OpcoesGrafico.UnidadeAbscissa.ToString());
            }
        }
        [DataMember]
        public List<PontoGrafico> SeriePrincipal
        {
            get
            {
                return this._seriePrincipal;
            }
            set
            {
                this._seriePrincipal = value;
                this.extrairDadosGraficoSeriePrincipal();
            }
        }
        [DataMember]
        public List<PontoGrafico> SerieModuloElasticidade
        {
            get
            {
                return this._serieModuloElasticidade;
            }
            set
            {
                this._serieModuloElasticidade = value;
                this.extrairDadosGraficoSerieModuloElasticidade();
            }
        }
        [DataMember]
        public List<PontoGrafico> SerieParalelaModulo
        {
            get
            {
                return this._serieParalelaModulo;
            }
            set
            {
                this._serieParalelaModulo = value;
                this.extrairDadosGraficoSerieParalelaModulo();
            }
        }
        [DataMember]
        public OpcoesGraficoTracao OpcoesGrafico
        {
            get
            {
                return this._opcoesGrafico;
            }
            set
            {
                this._opcoesGrafico = value;
            }
        }

        public List<Double> SeriePrincipalValoresAbscissa
        {
            set
            {
                this._seriePrincipalValoresAbscissa = value;
            }
            get
            {
                return this._seriePrincipalValoresAbscissa;
            }
        }
        public List<Double> SeriePrincipalValoresOrdenada
        {
            set
            {
                this._seriePrincipalValoresOrdenada = value;
            }
            get
            {
                return this._seriePrincipalValoresOrdenada;
            }
        }
        public List<Double> SerieModuloElasticidadeValoresAbscissa
        {
            set
            {
                this._serieModuloElasticidadeValoresAbscissa = value;
            }
            get
            {
                return this._serieModuloElasticidadeValoresAbscissa;
            }
        }
        public List<Double> SerieModuloElasticidadeValoresOrdenada
        {
            set
            {
                this._serieModuloElasticidadeValoresOrdenada = value;
            }
            get
            {
                return this._serieModuloElasticidadeValoresOrdenada;
            }
        }
        public List<Double> SerieParalelaModuloValoresAbscissa
        {
            set
            {
                this._serieParalelaModuloValoresAbscissa = value;
            }
            get
            {
                return this._serieParalelaModuloValoresAbscissa;
            }
        }
        public List<Double> SerieParalelaModuloValoresOrdenada
        {
            set
            {
                this._serieParalelaModuloValoresOrdenada = value;
            }
            get
            {
                return this._serieParalelaModuloValoresOrdenada;
            }
        }

        public GraficoEnsaioTracao()
        {
            this._seriePrincipal = new List<PontoGrafico>();
            this._serieModuloElasticidade = new List<PontoGrafico>();
            this._serieParalelaModulo = new List<PontoGrafico>();
            this._seriePrincipalValoresAbscissa = new List<Double>();
            this._seriePrincipalValoresOrdenada = new List<Double>();
            this._serieModuloElasticidadeValoresAbscissa = new List<Double>();
            this._serieModuloElasticidadeValoresOrdenada = new List<Double>();
            this._serieParalelaModuloValoresAbscissa = new List<Double>();
            this._serieParalelaModuloValoresOrdenada = new List<Double>();
            this._opcoesGrafico = new OpcoesGraficoTracao();
        }

        #endregion

        #region Titulos

        

        #endregion

        #region Eventos

        #region GraficoAlterado

        public delegate void GraficoChangedEvent(object sender, GraficoChangedEventArgs e);

        public event GraficoChangedEvent GraficoChanged;

        public class GraficoChangedEventArgs : EventArgs
        {
            public GraficoChangedEventArgs()
            {
            }
        }

        public void informGraficoChangedListeners()
        {
            if (GraficoChanged != null)
            {
                GraficoChanged(this, new GraficoChangedEventArgs());
            }
        }

        #endregion

        #endregion

        #region Serializacao

        [OnDeserialized]
        private void posDeserializacao(StreamingContext context)
        {
            this.extrairDadosGraficoSeriePrincipal();
            this.extrairDadosGraficoSerieModuloElasticidade();
        }

        #endregion

        #region Misc.

        private void extrairDadosGraficoSeriePrincipal()
        {
            //double maximoOrdenada = 1, maximoAbscissa = 0;
            List<Double> principalOrdenada = new List<Double>();
            List<Double> principalAbscissa = new List<Double>();

            foreach (PontoGrafico ponto in this.SeriePrincipal)
            {
                principalOrdenada.Add(ponto.ValorOrdenada);
                principalAbscissa.Add(ponto.ValorAbscissa);
                //maximoOrdenada = Math.Max((double)ponto.ValorOrdenada, maximoOrdenada);
                //maximoAbscissa = Math.Max((double)ponto.ValorAbscissa, maximoAbscissa);
            }

            //this.MaximoAbscissa = Math.Max(maximoAbscissa, obtemMaximoGrafico(maximoAbscissa));
            //this.MaximoOrdenada = Math.Max(maximoOrdenada, obtemMaximoGrafico(maximoOrdenada));
            this.SeriePrincipalValoresAbscissa = principalAbscissa;
            this.SeriePrincipalValoresOrdenada = principalOrdenada;
        }

        private void extrairDadosGraficoSerieModuloElasticidade()
        {
            List<Double> moduloOrdenada = new List<Double>();
            List<Double> moduloAbscissa = new List<Double>();

            foreach (PontoGrafico ponto in this.SerieModuloElasticidade)
            {
                moduloOrdenada.Add(ponto.ValorOrdenada);
                moduloAbscissa.Add(ponto.ValorAbscissa);
            }

            this.SerieModuloElasticidadeValoresAbscissa = moduloAbscissa;
            this.SerieModuloElasticidadeValoresOrdenada = moduloOrdenada;
        }

        private void extrairDadosGraficoSerieParalelaModulo()
        {
            List<Double> paralelaOrdenada = new List<Double>();
            List<Double> paralelaAbscissa = new List<Double>();

            foreach (PontoGrafico ponto in this.SerieParalelaModulo)
            {
                paralelaOrdenada.Add(ponto.ValorOrdenada);
                paralelaAbscissa.Add(ponto.ValorAbscissa);
            }

            this.SerieParalelaModuloValoresAbscissa = paralelaAbscissa;
            this.SerieParalelaModuloValoresOrdenada = paralelaOrdenada;
        }

        private static double obtemMaximoGrafico(double valor)
        {
            double expoente = Math.Floor(Math.Log10(Math.Abs(valor)));
            double mantissa = Math.Abs(valor) / Math.Pow(10, expoente);
            double sinal = 1.0;
            if (valor < 0.0)
            {
                sinal = -1.0;
            }

            double mantissaMax = 1.0;
            for (; mantissa > mantissaMax; mantissaMax += 0.2);

            return mantissaMax * Math.Pow(10, expoente) * sinal;
        }

        #endregion
    }
}
