﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Security.Cryptography;
using TestView.Seguranca;
using System.IO;
using TestView.Serializacao;
using System.Data.Sql;
using System.Data;

namespace TestView.Dados
{
    [DataContract]
    public class BancoDeDados
    {
        #region Construtor e membros

        private const string ARQUIVO_CONEXAO_BD = "./bd.xml";
        private static BancoDeDados _instancia;

        private Boolean _utilizarBancoDados;
        private String _nomeUsuario;
        private String _senha;
        private String _dataSource;

        [DataMember]
        public Boolean UtilizarBancoDados
        {
            get
            {
                return this._utilizarBancoDados;
            }
            set
            {
                this._utilizarBancoDados = value;
            }
        }
        [DataMember]
        public String NomeUsuario
        {
            get
            {
                return this._nomeUsuario;
            }
            set
            {
                this._nomeUsuario = value;
            }
        }
        public String Senha
        {
            get
            {
                if (this.SenhaSecreta != null && !this.SenhaSecreta.Equals(String.Empty))
                {
                    return Criptografia.getInstance().decriptar(this.SenhaSecreta);
                }
                else
                {
                    return this.SenhaSecreta;
                }
            }
            set
            {
                if (value != null && !value.Equals(String.Empty))
                {
                    this.SenhaSecreta = Criptografia.getInstance().encriptar(value);
                }
                else
                {
                    this.SenhaSecreta = value;
                }
            }
        }
        [DataMember]
        private String SenhaSecreta
        {
            get
            {
                return this._senha;
            }
            set
            {
                this._senha = value;
            }
        }
        [DataMember]
        public String DataSource
        {
            get
            {
                return this._dataSource;
            }
            set
            {
                this._dataSource = value;
            }
        }
        public String Catalog
        {
            get
            {
                return "testview";
            }
        }

        private BancoDeDados()
        {
            this.UtilizarBancoDados = false;
            this.SenhaSecreta = String.Empty;
            this.NomeUsuario = String.Empty;
            this.DataSource = String.Empty;
            this.carregaArquivoConfiguracao();
        }

        public static BancoDeDados getInstance()
        {
            if (_instancia == null)
            {
                _instancia = new BancoDeDados();
            }
            return _instancia;
        }

        #endregion

        #region Arquivo de configuração

        public void salvaArquivoConfiguracao(string nomeDoArquivo)
        {
            String caminhoCompleto = Path.GetFullPath(nomeDoArquivo);
            Stream stream = new FileStream(caminhoCompleto, FileMode.Create);
            Serializador.serializaObjeto(stream, this, typeof(BancoDeDados));
            stream.Close();
        }

        public void salvaArquivoConfiguracao()
        {
            this.salvaArquivoConfiguracao(ARQUIVO_CONEXAO_BD);
        }

        public void carregaArquivoConfiguracao(string nomeDoArquivo)
        {
            if (File.Exists(nomeDoArquivo))
            {
                String caminhoCompleto = Path.GetFullPath(nomeDoArquivo);
                Stream stream = new FileStream(caminhoCompleto, FileMode.Open);
                BancoDeDados bd = (BancoDeDados)Serializador.deserializaObjeto(stream, typeof(BancoDeDados));
                this.salvaBancoDados(bd.UtilizarBancoDados, bd.NomeUsuario, bd.Senha, bd.DataSource);
                stream.Close();
            }
        }

        public void carregaArquivoConfiguracao()
        {
            this.carregaArquivoConfiguracao(ARQUIVO_CONEXAO_BD);
        }

        #endregion

        #region Salvar BD

        public void salvaBancoDados(Boolean utilizarBd, String nomeUsuario, String senha, String instancia)
        {
            this.DataSource = instancia;
            this.NomeUsuario = nomeUsuario;
            this.Senha = senha;
            this.UtilizarBancoDados = utilizarBd;
            this.informBancoDadosListeners();
        }

        #endregion

        #region SQL Server

        public static List<String> listarInstanciasSqlServer()
        {
            DataTable resultado = SqlDataSourceEnumerator.Instance.GetDataSources();
            List<String> instancias = new List<String>();
            for (int i = 0; i < resultado.Rows.Count; i++)
            {
                if (resultado.Rows[i]["ServerName"] != DBNull.Value)
                {
                    String nomeServidor = (String)resultado.Rows[i]["ServerName"];
                    if (resultado.Rows[i]["InstanceName"] != DBNull.Value)
                    {
                        String nomeInstancia = (String)resultado.Rows[i]["InstanceName"];
                        String nomeCompletoInstancia = String.Format(@"{0}\{1}", nomeServidor, nomeInstancia);
                        instancias.Add(nomeCompletoInstancia);
                        continue;
                    }
                    instancias.Add(nomeServidor);
                    continue;
                }
            }
            return instancias;
        }

        #endregion

        #region Eventos

        #region ListaConexoesChanged

        public delegate void BancoDadosChangedEvent(object sender, BancoDadosEventArgs e);

        public event BancoDadosChangedEvent BancoDadosChanged;

        public class BancoDadosEventArgs : EventArgs
        {
            public BancoDadosEventArgs(BancoDeDados bd)
            {
                this.BancoDeDados = bd;
            }

            public BancoDeDados BancoDeDados
            {
                get;
                set;
            }
        }

        private void informBancoDadosListeners()
        {
            if (BancoDadosChanged != null)
            {
                BancoDadosChanged(this, new BancoDadosEventArgs(this));
            }
        }

        #endregion

        #endregion
    }
}
