﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using TestView.Seguranca;

namespace TestView.Ensaios
{
    public class CarregadorDeModulos
    {
        #region Construtor e membros

        private static CarregadorDeModulos _instance;
        private Dictionary<Modulo, BibliotecaModulo> _bibliotecasCarregadas;
        private static readonly Dictionary<Modulo, BibliotecaModulo> mapaBibliotecas = new Dictionary<Modulo, BibliotecaModulo> {
            {Modulo.ELASTICIDADE, BibliotecaModulo.BIBLIOTECA_ELASTICIDADE},
            {Modulo.TRACAO, BibliotecaModulo.BIBLIOTECA_TRACAO},
        };

        public Dictionary<Modulo, BibliotecaModulo> BibliotecasCarregadas
        {
            get
            {
                return this._bibliotecasCarregadas;
            }
        }

        private CarregadorDeModulos() 
        {
            this._bibliotecasCarregadas = new Dictionary<Modulo, BibliotecaModulo>();
        }

        public static CarregadorDeModulos getInstance()
        {
            if (_instance == null)
            {
                _instance = new CarregadorDeModulos();
            }
            return _instance;
        }

        #endregion

        #region Carregador

        public void carregaModulos()
        {
            this._bibliotecasCarregadas.Clear();
            foreach (KeyValuePair<Modulo, BibliotecaModulo> par in mapaBibliotecas)
            {
                BibliotecaModulo biblioteca = par.Value;
                Modulo modulo = par.Key;
                try
                {
                    Logging.getInstance().info(String.Format("Carregando biblioteca {0}.", biblioteca.NomeBiblioteca));
                    biblioteca.carregaBiblioteca();
                    this._bibliotecasCarregadas.Add(modulo, biblioteca);
                    Logging.getInstance().info("Carregado com sucesso.");
                }
                catch (Exception ex)
                {
                    Logging.getInstance().warn(String.Format("Impossivel carregar biblioteca {0}.", biblioteca.NomeBiblioteca), ex);
                }
            }
            this.informBibliotecasCarregadasListeners();
        }

        #endregion

        #region Eventos

        #region BibliotecasCarregadasChanged

        public delegate void BibliotecasCarregadasEvent(object sender, BibliotecasCarregadasEventArgs e);

        public event BibliotecasCarregadasEvent BibliotecasCarregadasChanged;

        public class BibliotecasCarregadasEventArgs : EventArgs
        {
            public BibliotecasCarregadasEventArgs(Dictionary<Modulo, BibliotecaModulo> bibliotecas)
            {
                this.BibliotecasCarregadas = bibliotecas;
            }

            public Dictionary<Modulo, BibliotecaModulo> BibliotecasCarregadas
            {
                get;
                set;
            }
        }

        private void informBibliotecasCarregadasListeners()
        {
            if (BibliotecasCarregadasChanged != null)
            {
                BibliotecasCarregadasChanged(this, new BibliotecasCarregadasEventArgs(this.BibliotecasCarregadas));
            }
        }

        #endregion

        #endregion
    }

    public enum Modulo
    {
        ELASTICIDADE,
        TRACAO,
    }
}
