﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace TestView.Ensaios.Utilidades.Coleta
{
    [DataContract]
    public class Aquisicao
    {
        #region Construtor e membros

        private List<SerieColeta> _seriesTemporais;

        [DataMember]
        public List<SerieColeta> SeriesTemporais
        {
            get
            {
                return this._seriesTemporais;
            }
            set
            {
                this._seriesTemporais = value;
            }
        }

        public int QuantidadePontosColetados
        {
            get
            {
                if (this.SeriesTemporais.Count > 0)
                {
                    return this.SeriesTemporais[0].PontosColetados.Count;
                }
                else
                {
                    return 0;
                }
            }
        }

        public Aquisicao()
        {
            this._seriesTemporais = new List<SerieColeta>();
        }

        #endregion

        #region Coleta

        public void adicionaSerieTemporal(SerieColeta serie)
        {
            this._seriesTemporais.Add(serie);
        }

        public void adicionaConjuntoPontos(DateTime hora, List<Tuple<SerieColeta, Double>> coleta)
        {
            foreach (Tuple<SerieColeta, Double> tuplaColeta in coleta)
            {
                SerieColeta serie = tuplaColeta.Item1;
                Double valor = tuplaColeta.Item2;
                serie.adicionaPonto(hora, valor);
            }
            this.informConjuntoDePontosAdicionadoListeners(hora, coleta);
        }

        #endregion

        #region Eventos

        #region PontoAdicionado

        public delegate void ConjuntoDePontosAdicionadoEvent(object sender, ConjuntoDePontosAdicionadoEventArgs e);

        public event ConjuntoDePontosAdicionadoEvent ConjuntoDePontosAdicionado;

        public class ConjuntoDePontosAdicionadoEventArgs : EventArgs
        {
            public ConjuntoDePontosAdicionadoEventArgs(DateTime horaColeta, List<Tuple<SerieColeta, Double>> coleta)
            {
                this.HoraColeta = horaColeta;
                this.Coleta = coleta;
            }

            public DateTime HoraColeta
            {
                get;
                set;
            }

            public List<Tuple<SerieColeta, Double>> Coleta
            {
                get;
                set;
            }
        }

        private void informConjuntoDePontosAdicionadoListeners(DateTime horaColeta, List<Tuple<SerieColeta, Double>> coleta)
        {
            if (ConjuntoDePontosAdicionado != null)
            {
                ConjuntoDePontosAdicionado(this, new ConjuntoDePontosAdicionadoEventArgs(horaColeta, coleta));
            }
        }

        #endregion

        #endregion

        #region Serializacao

        [OnDeserializing]
        private void inicializaMembros(StreamingContext context)
        {
            this._seriesTemporais = new List<SerieColeta>();
        }

        #endregion
    }
}
