﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Conexoes;
using System.Timers;
using TestView.Excecoes;
using TestView.Utils;
using TestView.Seguranca;

namespace TestView.Indicacao
{
    public class Indicador
    {
        #region Construtor e membros

        private Conexao _conexao;
        private long _indicacao;
        private DateTime _horarioColeta;
        private double _indicacaoAjustada;
        private string _indicacaoAjustadaString;
        private bool _inicializado;
        private double _percentual;
        private Temporizador _temporizador;

        public Conexao Conexao
        {
            get
            {
                return this._conexao;
            }
        }

        public long Indicacao
        {
            get
            {
                return this._indicacao;
            }
        }

        public DateTime HorarioColeta
        {
            get
            {
                return this._horarioColeta;
            }
        }

        public double IndicacaoAjustada
        {
            get
            {
                return this._indicacaoAjustada;
            }
        }

        public string IndicacaoAjustadaStr
        {
            get
            {
                return this._indicacaoAjustadaString;
            }
        }

        public double IndicacaoRelativa
        {
            get
            {
                return this._percentual;
            }
        }

        public bool Inicializado
        {
            get
            {
                return this._inicializado;
            }
        }

        public Indicador(Conexao conexao)
        {
            this._conexao = conexao;
            this._inicializado = false;
            //TODO: Remover intervalo padrao hardcoded
            this._temporizador = new Temporizador(20, new Temporizador.FuncaoTemporizador(Temporizador_Elapsed));
        }
        #endregion

        #region Eventos

        public delegate void IndicacaoChangedEvent(object sender, IndicacaoEventArgs e);

        public event IndicacaoChangedEvent IndicacaoChanged;

        public class IndicacaoEventArgs : EventArgs
        {
            public IndicacaoEventArgs(long indicacao, double indicacaoAjustada, string indicacaoAjustadaStr, double indicacaoRelativa)
            {
                this.Indicacao = indicacao;
                this.IndicacaoAjustada = indicacaoAjustada;
                this.IndicacaoAjustadaStr = indicacaoAjustadaStr;
                this.IndicacaoRelativa = indicacaoRelativa;
            }

            public long Indicacao
            {
                get;
                set;
            }

            public double IndicacaoAjustada
            {
                get;
                set;
            }

            public string IndicacaoAjustadaStr
            {
                get;
                set;
            }

            public double IndicacaoRelativa
            {
                get;
                set;
            }
        }

        private void informListeners()
        {
            if (IndicacaoChanged != null)
            {
                IndicacaoChanged(this, new IndicacaoEventArgs(Indicacao, IndicacaoAjustada, IndicacaoAjustadaStr, IndicacaoRelativa));
            }
        }

        #endregion

        #region Indicador

        public void iniciaIndicador() 
        {
            if (!Inicializado)
            {
                this._inicializado = true;
                this._temporizador.inicia();
            }
        }

        public void paraIndicador()
        {
            if (Inicializado)
            {
                this._temporizador.para();
                this._inicializado = false;
            }
        }

        private void Temporizador_Elapsed()
        {
            try
            {
                coletaIndicacao();
            }
            catch (Exception ex)
            {
                this.paraIndicador();
                Logging.getInstance().error(ex.Message, ex, false);
            }
        }

        private void coletaIndicacao()
        {
            long indicacao = this._conexao.getIndicacao();
            this._horarioColeta = DateTime.Now;
            this._indicacao = indicacao;
            this._indicacaoAjustada = this._conexao.CondicionadorSinal.Calibracao.obtemIndicacaoAjustada(this.Indicacao);
            this._indicacaoAjustadaString = this.IndicacaoAjustada.ToString(String.Format("N{0}", this.Conexao.CondicionadorSinal.Calibracao.PontoDecimal));
            this._percentual = this.IndicacaoAjustada / this.Conexao.CondicionadorSinal.Calibracao.Capacidade;
            this.informListeners();
        }

        #endregion
    }
}
