﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TestView.MyMath
{
    public class MinimosQuadrados
    {
        private double[,] _matrizA;
        private double[] _vetorB;
        private double[] _resultado;

        public MinimosQuadrados(double [,] matrizA, double[] vetorB)
        {
            this._matrizA = (double[,])matrizA.Clone();
            this._vetorB = (double[])vetorB.Clone();
            this._resultado = null;
        }

        public double[] resolucao()
        {
            if (this._resultado == null)
            {
                this.fatoracaoQRHouseholder();
                this._resultado = this.substituicaoParaTras(this._matrizA, this._vetorB);
            }
            return this._resultado;
        }

        private double[] substituicaoParaTras(double[,] matrizA, double[] b)
        {
            int n = matrizA.GetLength(1);
            double[] x = new double[n];
            for (int i = n - 1; i >= 0; i--)
            {
                x[i] = b[i];
                for (int j = i + 1; j < n; j++)
                {
                    x[i] -= matrizA[i,j] * x[j];
                }
                x[i] /= matrizA[i, i];
            }
            return x;
        }

        private void fatoracaoQRHouseholder()
        {
            int n = this._matrizA.GetLength(1);
            int m = this._matrizA.GetLength(0);
            for (int k = 0; k < n; k++)
            {
                double[] x = new double[m-k];
                for (int i = k; i < m; i++)
                {
                    x[i-k] = this._matrizA[i,k];
                }
                double normaX = Vetor.norma2(x);
                int sinal = Math.Sign(x[0]);

                double[] vk = new double[m-k];
                x.CopyTo(vk, 0);
                vk[0] = vk[0] + sinal * normaX;
                double normaVk = Vetor.norma2(vk);
                Vetor.multiplicaVetorEscalar(vk, 1.0 / normaVk);

                for (int j = k; j < n; j++)
                {
                    double c = 0.0;
                    for (int i = k; i < m; i++)
                    {
                        c += vk[i - k] * this._matrizA[i,j];
                    }
                    for (int i = k; i < m; i++)
                    {
                        this._matrizA[i, j] -= 2.0 * vk[i - k] * c;
                    }
                }

                double d = 0.0;
                for (int i = k; i < m; i++)
                {
                    d += vk[i - k] * this._vetorB[i];
                }
                for (int i = k; i < m; i++)
                {
                    this._vetorB[i] -= 2.0 * vk[i - k] * d;
                }
            }
        }
    }
}
