﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.IO;

namespace TestView.MyMath
{
    [DataContract]
    public class Unidade
    {
        public static readonly Unidade KGF = new Unidade(0, "kgf");
        public static readonly Unidade TF = new Unidade(1, "tf");
        public static readonly Unidade GF = new Unidade(2, "gf");
        public static readonly Unidade N = new Unidade(3, "N");
        public static readonly Unidade DAN = new Unidade(4, "daN");
        public static readonly Unidade KN = new Unidade(5, "kN");
        public static readonly Unidade LBF = new Unidade(6, "lbf");
        public static readonly Unidade DYN = new Unidade(7, "dyn");

        public static readonly Unidade KGF_S = new Unidade(8, "kgf/s");
        public static readonly Unidade TF_S = new Unidade(9, "tf/s");
        public static readonly Unidade N_S = new Unidade(10, "N/s");
        public static readonly Unidade KN_S = new Unidade(11, "kN/s");

        public static readonly Unidade MM = new Unidade(12, "mm");
        public static readonly Unidade PERCENTUAL_EXT = new Unidade(13, "% extensométrico");

        public static readonly Unidade MM_S = new Unidade(14, "mm/s");

        public static readonly Unidade SEGUNDOS = new Unidade(15, "s");

        public static readonly Unidade MPA = new Unidade(16, "MPa");
        public static readonly Unidade PSI = new Unidade(17, "psi");
        public static readonly Unidade KGF_MM2 = new Unidade(18, "kgf/mm²");
        public static readonly Unidade N_MM2 = new Unidade(19, "N/mm²");
        public static readonly Unidade GPA = new Unidade(20, "GPa");
        public static readonly Unidade PA = new Unidade(21, "Pa");
        public static readonly Unidade KPA = new Unidade(22, "kPa");

        public static readonly Unidade MPA_S = new Unidade(23, "MPa/s");

        public static readonly Unidade MM2 = new Unidade(24, "mm²");

        public static readonly Dictionary<int, Unidade> MapaUnidades = new Dictionary<int, Unidade>
        {
            {KGF.Id, KGF},
            {TF.Id, TF},
            {GF.Id, GF},
            {N.Id, N},
            {DAN.Id, DAN},
            {KN.Id, KN},
            {LBF.Id, LBF},
            {DYN.Id, DYN},
            {MM.Id, MM},
            {PERCENTUAL_EXT.Id, PERCENTUAL_EXT},
            {SEGUNDOS.Id, SEGUNDOS},
            {MPA.Id, MPA},
            {PSI.Id, PSI},
            {KGF_MM2.Id, KGF_MM2},
            {N_MM2.Id, N_MM2},
            {MM2.Id, MM2},
            {KGF_S.Id, KGF_S},
            {N_S.Id, N_S},
            {TF_S.Id, TF_S},
            {KN_S.Id, KN_S},
            {MM_S.Id, MM_S},
            {MPA_S.Id, MPA_S},
        };

        private int _id;
        private string _unidade;

        [DataMember]
        public int Id
        {
            get
            {
                return this._id;
            }
            set
            {
                this._id = value;
            }
        }

        public string Nome
        {
            get
            {
                return this._unidade;
            }
            set
            {
                this._unidade = value;
            }
        }

        private Unidade(int id, string unidade)
        {
            this._id = id;
            this._unidade = unidade;
        }

        public override string ToString()
        {
            return this._unidade;
        }

        public override bool Equals(object obj)
        {
            if (typeof(Unidade).Equals(obj.GetType()))
            {
                Unidade objUnidade = (Unidade)obj;
                return this._id.Equals(objUnidade._id);
            }
            else
            {
                return false;
            }
        }

        public static Unidade[] listaUnidadesDeForca()
        {
            return new Unidade[] { KGF, TF, GF, N, DAN, KN, LBF, DYN };
        }

        public static Unidade[] listaUnidadesDeDeformacao()
        {
            return new Unidade[] { MM, PERCENTUAL_EXT };
        }

        public static Unidade[] listaUnidadesDeDeslocamento()
        {
            return new Unidade[] { MM };
        }

        public static Unidade[] listaUnidadesDeTempo()
        {
            return new Unidade[] { SEGUNDOS };
        }

        public static Unidade[] listaUnidadesDeTensao()
        {
            return new Unidade[] { MPA, PSI, KGF_MM2, N_MM2 };
        }
    }
}
