﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using log4net;

namespace TestView.Seguranca
{
    public class Logging
    {
        #region Construtor e membros

        private static Logging logging = null;
        private string configFileFullPath = null;
        private ILog log = null;

        public static Logging getInstance()
        {
            if (logging == null)
            {
                logging = new Logging();
            }
            return logging;
        }

        private Logging()
        {
            string assemblyPath = Path.GetDirectoryName(Assembly.GetAssembly(typeof(Logging)).Location);
            string configFileName = "log4net.config.xml";
            this.configFileFullPath = Path.Combine(assemblyPath, configFileName);
            log4net.Config.XmlConfigurator.ConfigureAndWatch(new FileInfo(this.configFileFullPath));
            this.log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        }

        #endregion

        #region Logging

        public void debug(object mensagem, bool informListeners)
        {
            this.log.Debug(mensagem);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.DEBUG, mensagem.ToString());
            }
        }

        public void debug(object mensagem, Exception ex, bool informListeners)
        {
            this.log.Debug(mensagem, ex);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.DEBUG, mensagem.ToString(), ex);
            }
        }

        public void info(object mensagem, bool informListeners)
        {
            this.log.Info(mensagem);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.INFO, mensagem.ToString());
            }
        }

        public void info(object mensagem, Exception ex, bool informListeners)
        {
            this.log.Info(mensagem, ex);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.INFO, mensagem.ToString(), ex);
            }
        }

        public void warn(object mensagem, bool informListeners)
        {
            this.log.Warn(mensagem);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.ALERTA, mensagem.ToString());
            }
        }

        public void warn(object mensagem, Exception ex, bool informListeners)
        {
            this.log.Warn(mensagem, ex);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.ALERTA, mensagem.ToString(), ex);
            }
        }

        public void error(object mensagem, bool informListeners)
        {
            this.log.Error(mensagem);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.ERRO, mensagem.ToString());
            }
        }

        public void error(object mensagem, Exception ex, bool informListeners)
        {
            this.log.Error(mensagem, ex);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.ERRO, mensagem.ToString(), ex);
            }
        }

        public void fatal(object mensagem, bool informListeners) {
            this.log.Fatal(mensagem);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.FATAL, mensagem.ToString());
            }
        }

        public void fatal(object mensagem, Exception ex, bool informListeners)
        {
            this.log.Fatal(mensagem, ex);
            if (informListeners)
            {
                this.informOnLoggingListeners(NivelLog.FATAL, mensagem.ToString(), ex);
            }
        }

        #endregion

        #region Eventos

        #region OnLogging

        public delegate void OnLoggingEvent(object sender, OnLoggingEventArgs e);

        public event OnLoggingEvent OnLogging;

        public class OnLoggingEventArgs : EventArgs
        {
            public OnLoggingEventArgs(NivelLog nivel, String mensagem, Exception ex)
            {
                this.Nivel = nivel;
                this.Mensagem = mensagem;
                this.Excecao = ex;
            }

            public NivelLog Nivel
            {
                get;
                set;
            }

            public String Mensagem
            {
                get;
                set;
            }

            public Exception Excecao
            {
                get;
                set;
            }
        }

        private void informOnLoggingListeners(NivelLog nivel, String mensagem, Exception ex)
        {
            if (OnLogging != null)
            {
                OnLogging(this, new OnLoggingEventArgs(nivel, mensagem, ex));
            }
        }

        private void informOnLoggingListeners(NivelLog nivel, String mensagem)
        {
            this.informOnLoggingListeners(nivel, mensagem, null);
        }

        #endregion

        #endregion
    }

    public enum NivelLog
        {
            DEBUG,
            INFO,
            ALERTA,
            ERRO,
            FATAL
        }
}
