﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Conexoes;
using TestView;

namespace TestViewGUI.Calibracao
{
    public partial class FormArquivosDeCalibracao : Form
    {
        #region Construtor e membros

        private int _indice;
        private Dictionary<Panel, Conexao> _dicionarioPainelConexao;

        public FormArquivosDeCalibracao()
        {
            this._indice = 0;
            this._dicionarioPainelConexao = new Dictionary<Panel, Conexao>();
            InitializeComponent();
        }

        #endregion

        #region Layout

        private Panel criaNovoPanelArquivoCalibracao(string descricaoConexao, string caminhoDoArquivo)
        {
            Panel panel = new Panel();
            panel.Name = String.Format("panelArquivoCalibracao{0}", _indice);
            panel.Size = new Size(groupBoxArquivosCalibracao.Size.Width-12, 32);
            panel.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            panel.Location = new Point(6, 19 + _dicionarioPainelConexao.Count * 28);

            Label labelDescricaoConexao = new Label();
            labelDescricaoConexao.Name = String.Format("labelDescricaoConexao{0}", _indice);
            labelDescricaoConexao.Text = descricaoConexao;
            labelDescricaoConexao.AutoSize = true;
            labelDescricaoConexao.Location = new Point(3, 9);

            TextBox textBoxCaminhoArquivo = new TextBox();
            textBoxCaminhoArquivo.Name = String.Format("textBoxCaminhoArquivo{0}", _indice);
            textBoxCaminhoArquivo.Text = caminhoDoArquivo;
            textBoxCaminhoArquivo.ReadOnly = true;
            textBoxCaminhoArquivo.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            textBoxCaminhoArquivo.Size = new Size(panel.Size.Width-204, 20);
            textBoxCaminhoArquivo.Location = new Point(100,6);

            Button buttonProcurar = new Button();
            buttonProcurar.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            buttonProcurar.Name = String.Format("buttonProcurar{0}", _indice);
            buttonProcurar.Text = "Procurar...";
            buttonProcurar.Location = new Point(panel.Size.Width - 98, 6);
            buttonProcurar.Size = new Size(95, 20);
            buttonProcurar.Click += new EventHandler(buttonProcurar_Click);

            panel.Controls.Add(labelDescricaoConexao);
            panel.Controls.Add(textBoxCaminhoArquivo);
            panel.Controls.Add(buttonProcurar);
            return panel;
        }

        private void adicionaNovoPanelArquivoCalibracao(Conexao conexao)
        {
            String descricaoConexao = conexao.ToString();
            String caminhoDoArquivo = conexao.CondicionadorSinal.ArquivoCalibracao;
            Panel panel = criaNovoPanelArquivoCalibracao(descricaoConexao, caminhoDoArquivo);
            _dicionarioPainelConexao.Add(panel, conexao);
            inserePanelArquivoCalibracao(panel);
            _indice++;
        }

        private void inserePanelArquivoCalibracao(Panel panel)
        {
            groupBoxArquivosCalibracao.Controls.Add(panel);
            this.Size = new Size(this.Size.Width, this.Size.Height + 28);
        }

        private TextBox obtemTextBoxApartirDoPanel(Panel panel)
        {
            TextBox textBoxArquivoCalibracao = null;
            foreach (Control control in panel.Controls)
            {
                if (typeof(TextBox).Equals(control.GetType()))
                {
                    textBoxArquivoCalibracao = (TextBox)control;
                }
            }
            return textBoxArquivoCalibracao;
        }

        private void coletaCaminhoArquivoDeCalibracao(Button buttonProcurar)
        {
            Panel panelArquivoCalibracao = (Panel)buttonProcurar.Parent;
            TextBox textBoxArquivoCalibracao = obtemTextBoxApartirDoPanel(panelArquivoCalibracao);
            OpenFileDialog abrirArquivoCalibracaoDialog = new OpenFileDialog();
            abrirArquivoCalibracaoDialog.ShowDialog(this);
            if (abrirArquivoCalibracaoDialog.FileName != String.Empty && abrirArquivoCalibracaoDialog.CheckFileExists)
            {
                textBoxArquivoCalibracao.Text = abrirArquivoCalibracaoDialog.FileName;
            }
        }

        #endregion

        #region Dados

        private void salvaCaminhosArquivosDeCalibracao()
        {
            foreach (Control control in groupBoxArquivosCalibracao.Controls)
            {
                if (typeof(Panel).Equals(control.GetType()))
                {
                    Panel panel = (Panel) control;
                    TextBox textBoxArquivoCalibracao = this.obtemTextBoxApartirDoPanel(panel);
                    String caminhoArquivoCalibracao = textBoxArquivoCalibracao.Text;
                    Conexao conexao = this._dicionarioPainelConexao[panel];
                    conexao.CondicionadorSinal.ArquivoCalibracao = caminhoArquivoCalibracao;
                    conexao.CondicionadorSinal.carregarArquivoDeCalibracao();
                }
            }
        }

        #endregion

        #region Eventos do form

        private void FormArquivosDeCalibracao_Load(object sender, EventArgs e)
        {
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes())
            {
                adicionaNovoPanelArquivoCalibracao(conexao);
            }
        }

        private void buttonProcurar_Click(object sender, EventArgs e)
        {
            Button buttonProcurar = (Button)sender;
            this.coletaCaminhoArquivoDeCalibracao(buttonProcurar);
        }

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            this.salvaCaminhosArquivosDeCalibracao();
            this.Close();
        }

        #endregion
    }
}
