﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Conexoes;
using TestView;
using TestView.Indicacao;
using TestView.Condicionador.ItemCalibracao;
using TestView.Condicionador;
using TestView.MyMath;

namespace TestViewGUI.Calibracao
{
    public partial class FormCalibracao : Form
    {
        #region Construtor e membros

        private SaveFileDialog _salvarCalibracaoDialog;

        public FormCalibracao()
        {
            InitializeComponent();
            preencheComboboxConexoes();
            preencheComboboxResolucao();
            preencheComboboxTransdutor();
            preencheComboboxBaudRate();
            habilitaInterface(false);
            inicializaSaveFileDialog();
        }

        #endregion

        #region Diálogo para Salvar arquivo de calibração

        private void inicializaSaveFileDialog()
        {
            this._salvarCalibracaoDialog = new SaveFileDialog();
            this._salvarCalibracaoDialog.FileOk += new CancelEventHandler(salvarCalibracaoDialog_FileOk);
        }

        void salvarCalibracaoDialog_FileOk(object sender, CancelEventArgs e)
        {
            Calibrador.getInstance().salvarArquivoCalibracao(this._salvarCalibracaoDialog.FileName);
        }

        #endregion

        #region Preenchimento do Form

        private void preencheComboboxConexoes()
        {
            ComboBoxConexao.Items.Clear();
            List<Conexao> conexoes = Calibrador.getInstance().listaConexoes();
            ComboBoxConexao.Items.AddRange(conexoes.ToArray());
            if (ComboBoxConexao.Items.Count > 0)
            {
                ComboBoxConexao.SelectedIndex = 0;
            }
        }

        private void preencheComboboxResolucao()
        {
            ComboBoxResolucao.Items.Clear();
            foreach (int resolucao in TestView.Condicionador.Calibracao.listaResolucao()) {
                ComboBoxResolucao.Items.Add(resolucao);
            }
            ComboBoxResolucao.SelectedItem = TestView.Condicionador.Calibracao.ResolucaoPadrao;
        }

        private void preencheComboboxBaudRate()
        {
            comboBoxBaudRate.Items.Clear();
            foreach (int baudrate in TestView.Conexoes.PortaSerial.listaBaudRates())
            {
                comboBoxBaudRate.Items.Add(baudrate);
            }
            comboBoxBaudRate.SelectedItem = TestView.Conexoes.PortaSerial.BaudRatePadrao;
        }

        private void preencheComboboxTransdutor()
        {
            ComboBoxTransdutor.Items.Clear();
            ComboBoxTransdutor.Items.AddRange(new Transdutor[]{ Transdutor.FORCA, Transdutor.DEFORMACAO, Transdutor.DESLOCAMENTO });
        }

        private void preencheeComboboxUnidade(Transdutor tipoTransdutor)
        {
            ComboBoxUnidade.Items.Clear();
            if (tipoTransdutor.Equals(Transdutor.FORCA))
            {
                foreach (Unidade unidade in TestView.MyMath.Unidade.listaUnidadesDeForca())
                {
                    ComboBoxUnidade.Items.Add(unidade);
                }
            }
            else if (tipoTransdutor.Equals(Transdutor.DEFORMACAO))
            {
                foreach (Unidade unidade in TestView.MyMath.Unidade.listaUnidadesDeDeformacao())
                {
                    ComboBoxUnidade.Items.Add(unidade);
                }
            }
            else if (tipoTransdutor.Equals(Transdutor.DESLOCAMENTO))
            {
                foreach (Unidade unidade in TestView.MyMath.Unidade.listaUnidadesDeDeslocamento())
                {
                    ComboBoxUnidade.Items.Add(unidade);
                }
            }
        }

        private void habilitaInterface(Boolean conectado)
        {
            this.groupBoxConfiguracao.Enabled = conectado;
            this.groupBoxCalibracao.Enabled = conectado;
            this.ComboBoxConexao.Enabled = !conectado;
            this.buttonConectar.Enabled = !conectado;
            this.buttonDesconectar.Enabled = conectado;
            this.buttonSalvar.Enabled = conectado;
        }

        private void selecionaParametrosDeCalibracao()
        {
            this.ComboBoxPontoDecimal.SelectedIndex = Calibrador.getInstance().getPontoDecimal();
            this.ComboBoxResolucao.SelectedItem = Calibrador.getInstance().getResolucao();
            this.ComboBoxTransdutor.SelectedItem = Calibrador.getInstance().getTipoTransdutor();
            this.ComboBoxUnidade.SelectedItem = Calibrador.getInstance().getUnidade();
            this.textBoxCapacidade.Text = Calibrador.getInstance().getcapacidade().ToString();
        }

        private void selecionaConfiguracao()
        {
            this.ComboBoxFiltroDigital.SelectedIndex = Calibrador.getInstance().getFiltroDigital();
            this.comboBoxBaudRate.SelectedItem = Calibrador.getInstance().getBaudRate();
        }

        #endregion

        # region DataGrid Pontos De Calibração

        private void relacionaTabelaPontosCalibracao()
        {
            this.dataGridViewPontosCalibracao.DataSource = Calibrador.getInstance().listaPontosDeCalibracao();
        }

        private void pontosDeCalibracaoChanged(object sender, TestView.Condicionador.Calibracao.PontosDeCalibracaoEventArgs e)
        {
            this.dataGridViewPontosCalibracao.DataSource = Calibrador.getInstance().listaPontosDeCalibracao();
        }

        # endregion

        #region Indicacao

        private void indicacaoChanged(object sender, Indicador.IndicacaoEventArgs e)
        {
            long indicacao = e.Indicacao;
            string valorAjustado = e.IndicacaoAjustadaStr;
            Unidade unidade = Calibrador.getInstance().getUnidade();
            setLabelLeituraText(String.Format("{0:d} {1}", indicacao, Properties.Resources.pontosString));
            setLabelLeituraAjusteText(String.Format("{0} {1}", new object[]{valorAjustado, unidade}));
        }

        private delegate void SetLabelLeituraTextCallBack(string text);

        private void setLabelLeituraText(string text)
        {
            if (this.labelLeitura.InvokeRequired)
            {
                SetLabelLeituraTextCallBack d = new SetLabelLeituraTextCallBack(setLabelLeituraText);
                try
                {
                    this.Invoke(d, new object[] { text });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                this.labelLeitura.Text = text;
            }
        }

        private delegate void SetLabelLeituraAjusteTextCallBack(string text);

        private void setLabelLeituraAjusteText(string text)
        {
            if (this.labelLeituraAjuste.InvokeRequired)
            {
                SetLabelLeituraTextCallBack d = new SetLabelLeituraTextCallBack(setLabelLeituraAjusteText);
                try
                {
                    this.Invoke(d, new object[] { text });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                this.labelLeituraAjuste.Text = text;
            }
        }

        #endregion

        #region Calibração

        private void setFiltroDigital(int indiceFiltro)
        {
            Calibrador.getInstance().setFiltroDigital(indiceFiltro);
        }

        private void setTipoTransdutor(Transdutor transdutor)
        {
            Calibrador.getInstance().setTipoTransdutor(transdutor);
        }

        private void setUnidade(Unidade unidade)
        {
            Calibrador.getInstance().setUnidade(unidade);
        }

        private void setResolucao(int resolucao)
        {
            Calibrador.getInstance().setResolcuao(resolucao);
        }

        private void setPontoDecimal(int pontoDecimal)
        {
            Calibrador.getInstance().setPontoDecimal(pontoDecimal);
        }

        private void setCapacidade(String capacidadeString)
        {
            double capacidade = Double.Parse(capacidadeString);
            Calibrador.getInstance().setCapacidade(capacidade);
        }

        private Boolean validaCapacidade(String capacidadeString)
        {
            try
            {
                Double.Parse(capacidadeString);
                return true;
            }
            catch (FormatException)
            {
                return false;
            }
        }

        #endregion

        #region Conexão

        private void conectar()
        {
            if (ComboBoxConexao.SelectedItem != null)
            {
                Conexao conexao = (Conexao)ComboBoxConexao.SelectedItem;
                conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(this.Conexao_ConexaoChanged);
                Calibrador.getInstance().conectar(conexao);
                Calibrador.getInstance().IndicacaoChanged += new Calibrador.IndicacaoChangedEvent(this.indicacaoChanged);
                Calibrador.getInstance().PontosDeCalibracaoChanged += new TestView.Calibrador.PontosDeCalibracaoChangedEvent(pontosDeCalibracaoChanged);
                Calibrador.getInstance().ParametrosDeCalibracaoChanged += new Calibrador.ParametrosDeCalibracaoChangedEvent(parametrosDeCalibracaoChanged);
            }
        }

        void parametrosDeCalibracaoChanged(object sender, TestView.Condicionador.Calibracao.ParametrosDeCalibracaoEventArgs e)
        {
            this.selecionaParametrosDeCalibracao();
        }

        private void Conexao_ConexaoChanged(object sender, Conexao.ConexaoEventArgs e)
        {
            this.habilitaInterface(e.EstaConectado);
        }

        #endregion

        #region Eventos do Form

        private void buttonFechar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void FormCalibracao_Load(object sender, EventArgs e)
        {
            Calibrador.getInstance().desconectarTudo();
        }

        private void FormCalibracao_FormClosing(object sender, FormClosingEventArgs e)
        {
            Calibrador.getInstance().desconectar();
        }

        private void buttonConectar_Click(object sender, EventArgs e)
        {
            if (this.ComboBoxConexao.SelectedItem != null)
            {
                this.conectar();
                this.selecionaParametrosDeCalibracao();
                this.selecionaConfiguracao();
                this.relacionaTabelaPontosCalibracao();
            }
        }

        private void buttonDesconectar_Click(object sender, EventArgs e)
        {
            Calibrador.getInstance().desconectar();
        }

        private void buttonAlterarFiltroDigital_Click(object sender, EventArgs e)
        {
            this.setFiltroDigital(ComboBoxFiltroDigital.SelectedIndex);
        }

        private void ComboBoxTransdutor_SelectedIndexChanged(object sender, EventArgs e)
        {
            Transdutor transdutor = (Transdutor)ComboBoxTransdutor.SelectedItem;
            this.preencheeComboboxUnidade(transdutor);
            if (Calibrador.getInstance().getTipoTransdutor().Equals(transdutor))
            {
                if (this.ComboBoxUnidade.Items.Contains(Calibrador.getInstance().getUnidade()))
                {
                    this.ComboBoxUnidade.SelectedItem = Calibrador.getInstance().getUnidade();
                }
                else
                {
                    this.ComboBoxUnidade.SelectedIndex = 0;
                }
            }
            else
            {
                this.ComboBoxUnidade.SelectedIndex = 0;
            }
            this.setTipoTransdutor(transdutor);
        }

        private void ComboBoxUnidade_SelectedIndexChanged(object sender, EventArgs e)
        {
            labelUnidadeCapacidade.Text = ComboBoxUnidade.SelectedItem.ToString();
            setUnidade((Unidade)ComboBoxUnidade.SelectedItem);
        }

        private void textBoxCapacidade_Validating(object sender, CancelEventArgs e)
        {
            if (!validaCapacidade(textBoxCapacidade.Text))
            {
                e.Cancel = true;
                textBoxCapacidade.BackColor = Color.Tomato;
            }
            else
            {
                textBoxCapacidade.BackColor = SystemColors.Window;
            }
        }

        private void ComboBoxResolucao_SelectedIndexChanged(object sender, EventArgs e)
        {
            setResolucao((int)ComboBoxResolucao.SelectedItem);
        }

        private void ComboBoxPontoDecimal_SelectedIndexChanged(object sender, EventArgs e)
        {
            setPontoDecimal(ComboBoxPontoDecimal.SelectedIndex);
        }

        private void textBoxCapacidade_Validated(object sender, EventArgs e)
        {
            setCapacidade(textBoxCapacidade.Text);
        }

        private void buttonAdicionar_Click(object sender, EventArgs e)
        {
            long ponto = Calibrador.getInstance().getIndicacao();
            Unidade unidade = Calibrador.getInstance().getUnidade();
            FormAdicionaPonto formAdicionaPonto = new FormAdicionaPonto(unidade, ponto);
            formAdicionaPonto.ShowDialog(this);
        }

        private void buttonRemover_Click(object sender, EventArgs e)
        {
            if (dataGridViewPontosCalibracao.SelectedCells.Count > 0)
            {
                int indice = dataGridViewPontosCalibracao.SelectedCells[0].RowIndex;
                Calibrador.getInstance().removePontoCalibracao(indice);
            }
        }

        private void buttonZero_Click(object sender, EventArgs e)
        {
            Calibrador.getInstance().tara();
        }

        private void buttonSalvar_Click(object sender, EventArgs e)
        {
            this._salvarCalibracaoDialog.ShowDialog();
        }

        #endregion
    }
}
