﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Ensaios.Utilidades.Instrumentacao;
using TestView.Conexoes;
using TestView;

namespace TestViewGUI.Instrumentacao
{
    public partial class FormInstrumentacao : Form
    {
        #region Construtor e membros
        private Dictionary<Conexao, Instrumento> _instrumentacao;

        public FormInstrumentacao()
        {
            InitializeComponent();
            this.inicializaInstrumentacao();
            this.redesenhaFormularioInstrumentacao();
            this.habilitaBotoesDetalhes();
        }

        #endregion

        #region Layout

        private void redesenhaFormularioInstrumentacao()
        {
            this.groupBoxInstrumentos.Controls.Clear();
            int indiceControle = 0;
            foreach (KeyValuePair<Conexao, Instrumento> instrumentacao in this._instrumentacao)
            {
                Conexao conexao = instrumentacao.Key;
                Instrumento instrumento = instrumentacao.Value;
                this.adicionaOpcaoInstrumentacao(indiceControle, conexao, instrumento);
                indiceControle += 1;
            }
            this.Size = new Size(this.Width, 98 + indiceControle * 27);
        }

        private void habilitaBotoesDetalhes()
        {
            foreach (Control control in this.groupBoxInstrumentos.Controls)
            {
                if (control.Name.StartsWith("buttonDetalhes"))
                {
                    Button buttonDetalhes = (Button)control;
                    this.habilitaBotaoDetalhes(buttonDetalhes);
                }
            }
        }

        private void habilitaBotaoDetalhes(Button buttonDetalhes)
        {
            Conexao conexao = (Conexao)buttonDetalhes.Tag;
            Instrumento instrumento = this._instrumentacao[conexao];
            if (instrumento.GetType().Equals(typeof(Extensometro)))
            {
                buttonDetalhes.Enabled = true;
            }
            else
            {
                buttonDetalhes.Enabled = false;
            }
        }

        private void adicionaOpcaoInstrumentacao(int indice, Conexao conexao, Instrumento instrmento)
        {
            Label labelDescricaoConexao = new Label();
            labelDescricaoConexao.Name = String.Format("labelDescricaoConexao{0}", indice);
            labelDescricaoConexao.AutoSize = true;
            labelDescricaoConexao.Text = String.Format("{0} - {1}:", conexao.PortaSerial.NomePorta, conexao.CondicionadorSinal.Calibracao.TipoTransdutor.Descricao);
            labelDescricaoConexao.Location = new Point(11, 28 + indice * 27);

            ComboBox comboboxInstrumento = new ComboBox();
            comboboxInstrumento.Name = String.Format("comboboxInstrumento{0}", indice);
            comboboxInstrumento.Tag = conexao;
            comboboxInstrumento.DropDownStyle = ComboBoxStyle.DropDownList;
            comboboxInstrumento.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            comboboxInstrumento.Size = new Size(156, 21);
            comboboxInstrumento.Location = new Point(this.groupBoxInstrumentos.Size.Width - 240, 25 + indice * 27);
            comboboxInstrumento.Items.AddRange(Instrumento.Instrumentos.ToArray());
            foreach (Instrumento instCombobox in comboboxInstrumento.Items)
            {
                if (conexao.Instrumento.GetType().Equals(instCombobox.GetType()))
                {
                    comboboxInstrumento.SelectedItem = instCombobox;
                }
                if (comboboxInstrumento.SelectedItem == null && instCombobox.GetType().Equals(typeof(InstrumentoNaoEspecificado)))
                {
                    comboboxInstrumento.SelectedItem = instCombobox;
                }
            }
            comboboxInstrumento.SelectedIndexChanged += new EventHandler(comboboxInstrumento_SelectedIndexChanged);

            Button buttonDetalhes = new Button();
            buttonDetalhes.Name = String.Format("buttonDetalhes{0}", indice);
            buttonDetalhes.Tag = conexao;
            buttonDetalhes.Text = Properties.Resources.detalhesString;
            buttonDetalhes.Size = new Size(67, 21);
            buttonDetalhes.Anchor = AnchorStyles.Right | AnchorStyles.Top;
            buttonDetalhes.Location = new Point(this.groupBoxInstrumentos.Size.Width -  78, 25 + indice * 27);
            buttonDetalhes.Click += new EventHandler(buttonDetalhes_Click);

            this.groupBoxInstrumentos.Controls.Add(buttonDetalhes);
            this.groupBoxInstrumentos.Controls.Add(labelDescricaoConexao);
            this.groupBoxInstrumentos.Controls.Add(comboboxInstrumento);
        }

        #endregion

        #region Detalhes do instrumento

        private void abrirDetalhesInstrumento(Instrumento instrumento)
        {
            if (instrumento.GetType().Equals(typeof(Extensometro)))
            {
                Extensometro extensometro = (Extensometro)instrumento;
                FormExtensometro form = new FormExtensometro(extensometro);
                form.ShowDialog();
            }
        }

        #endregion

        #region Instrumentacao

        private void inicializaInstrumentacao()
        {
            this._instrumentacao = new Dictionary<Conexao, Instrumento>();
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes())
            {
                this._instrumentacao.Add(conexao, (Instrumento)conexao.Instrumento.Clone());
            }
        }

        private void salvaInstrumentacao()
        {
            foreach (KeyValuePair<Conexao, Instrumento> instrumentacao in this._instrumentacao)
            {
                Conexao conexao = instrumentacao.Key;
                Instrumento instrumento = instrumentacao.Value;

                conexao.Instrumento = (Instrumento)instrumento.Clone();
            }
        }

        #endregion

        #region Eventos

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            this.salvaInstrumentacao();
            this.Close();
        }

        private void comboboxInstrumento_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboboxInstrumento = (ComboBox)sender;
            Conexao conexao = (Conexao)comboboxInstrumento.Tag;

            Instrumento instrumentoSelecionado = (Instrumento)comboboxInstrumento.SelectedItem;
            if (!this._instrumentacao[conexao].GetType().Equals(instrumentoSelecionado.GetType()))
            {
                this._instrumentacao[conexao] = (Instrumento)instrumentoSelecionado.Clone();
                this.habilitaBotoesDetalhes();
            }
        }

        private void buttonDetalhes_Click(object sender, EventArgs e)
        {
            Button buttonDetalhes = (Button)sender;
            Conexao conexao = (Conexao)buttonDetalhes.Tag;
            Instrumento instrumento = this._instrumentacao[conexao];
            this.abrirDetalhesInstrumento(instrumento);
        }

        #endregion
    }
}
