﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView;
using System.Globalization;
using System.Threading;
using TestViewGUI.Conexoes;
using TestViewGUI.Calibracao;
using TestView.Conexoes;
using TestView.Indicacao;
using TestView.Seguranca;
using TestView.Ensaios;
using TestView.MyMath;
using TestViewGUI.Excecoes;
using TestView.Ensaios.Modulos;
using TestViewGUI.Instrumentacao;
using TestViewGUI.Dados;

namespace TestViewGUI
{
    public partial class FormPrincipal : Form
    {
        #region Construtor e membros

        private int _indiceTransdutores;
        private Dictionary<Conexao, GroupBox> _dicionarioConexaoGroupTransdutores;
        private Dictionary<GroupBox, Conexao> _dicionarioGroupTransdutoresConexao;
        private Dictionary<GroupBox, int> _dicionatioGroupTransdutoresIndice;
        private List<ToolStripMenuItem> _menusEnsaio;

        public FormPrincipal()
        {
            this._indiceTransdutores = 0;
            this._dicionarioConexaoGroupTransdutores = new Dictionary<Conexao, GroupBox>();
            this._dicionarioGroupTransdutoresConexao = new Dictionary<GroupBox, Conexao>();
            this._dicionatioGroupTransdutoresIndice = new Dictionary<GroupBox, int>();
            this._menusEnsaio = new List<ToolStripMenuItem>();
            InitializeComponent();
            GerenciadorAlerta.inicializaGerenciador();
            GerenciadorEnsaios.getInstance().ListaConexoesChanged += new GerenciadorEnsaios.ListaConexoesChangedEvent(listaConexoesChanged);
            CarregadorDeModulos.getInstance().BibliotecasCarregadasChanged += new CarregadorDeModulos.BibliotecasCarregadasEvent(bibliotecasCarregadasChanged);
            GerenciadorEnsaios.getInstance().ModuloEnsaioAdicionado += new GerenciadorEnsaios.EnsaioAdicionadoEvent(moduloEnsaioAdicionado);
            GerenciadorEnsaios.getInstance().ModuloEnsaioRemovido += new GerenciadorEnsaios.EnsaioRemovidoEvent(moduloEnsaioRemovido);
            this.atualizaPainelTransdutores();
            this.atualizaMenuNovoEnsaio();
        }

        #endregion

        #region Conexão

        private void alternaEstadoConexao(int indice)
        {
            if (indice >= 0 && indice < GerenciadorEnsaios.getInstance().listaConexoes().Count)
            {
                Conexao conexao = GerenciadorEnsaios.getInstance().listaConexoes()[indice];
                GerenciadorEnsaios.getInstance().alternarEstadoConexao(conexao);
            }
        }

        #endregion

        #region Painel de transdutores

        private void alternarPainelTransdutores()
        {
            exibirTransdutoresToolStripMenuItem.Checked = !exibirTransdutoresToolStripMenuItem.Checked;
            splitContainerPrincipal.Panel2Collapsed = !exibirTransdutoresToolStripMenuItem.Checked;
        }

        #region Inicializacao dos transdutores

        private GroupBox criaNovoGroupBoxTransdutor(Conexao conexao)
        {
            GroupBox groupBox = new GroupBox();
            groupBox.Name = String.Format(Properties.Resources.groupBoxTransdutorName, this._indiceTransdutores);
            groupBox.Text = TestView.Condicionador.Calibracao.dicionarioTipoTransdutor()[conexao.CondicionadorSinal.Calibracao.TipoTransdutor];
            groupBox.Size = new Size(209, 142);
            groupBox.Location = new Point(6, 19 + this._dicionarioConexaoGroupTransdutores.Count * 148);

            PictureBox pictureBoxBotaoConectar = new PictureBox();
            pictureBoxBotaoConectar.Name = String.Format(Properties.Resources.pictureBoxConectarTransdutorName, this._indiceTransdutores);
            pictureBoxBotaoConectar.Cursor = Cursors.Hand;
            pictureBoxBotaoConectar.Image = Properties.Resources.connect_green;
            pictureBoxBotaoConectar.SizeMode = PictureBoxSizeMode.AutoSize;
            pictureBoxBotaoConectar.Location = new Point(9, 19);
            pictureBoxBotaoConectar.Click += new EventHandler(pictureBoxBotaoConectar_Click);

            Label labelConectar = new Label();
            labelConectar.Name = String.Format(Properties.Resources.labelConectarTransdutorName, this._indiceTransdutores);
            labelConectar.Font = new Font(FontFamily.GenericSansSerif, 14.25F, FontStyle.Bold, GraphicsUnit.Point);
            labelConectar.ForeColor = Color.RoyalBlue;
            labelConectar.Cursor = Cursors.Hand;
            labelConectar.AutoSize = true;
            labelConectar.Location = new Point(35, 17);
            labelConectar.Click += new EventHandler(labelConectar_Click);

            ProgressBar progressBar = new ProgressBar();
            progressBar.Name = String.Format(Properties.Resources.progressBarTransdutorName, this._indiceTransdutores);
            progressBar.ForeColor = Color.Green;
            progressBar.Maximum = 200;
            progressBar.MarqueeAnimationSpeed = 200;
            progressBar.Step = 1;
            progressBar.Location = new Point(6, 76);
            progressBar.Size = new Size(175, 16);

            Label labelIndicacaoTransdutor = new Label();
            labelIndicacaoTransdutor.Name = String.Format(Properties.Resources.labelIndicacaoTransdutorName, this._indiceTransdutores);
            labelIndicacaoTransdutor.Font = new Font(FontFamily.GenericSansSerif, 14.25F, FontStyle.Bold, GraphicsUnit.Point);
            labelIndicacaoTransdutor.Text = Properties.Resources.indicacaoVaziaString;
            labelIndicacaoTransdutor.TextAlign = ContentAlignment.MiddleRight;
            labelIndicacaoTransdutor.Location = new Point(6, 42);
            labelIndicacaoTransdutor.Size = new Size(197, 33);
            labelIndicacaoTransdutor.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;

            PictureBox pictureBoxAlerta = new PictureBox();
            pictureBoxAlerta.Name = String.Format(Properties.Resources.pictureBoxAlertaName, this._indiceTransdutores);
            pictureBoxAlerta.Location = new Point(187, 76);
            pictureBoxAlerta.SizeMode = PictureBoxSizeMode.AutoSize;
            pictureBoxAlerta.Image = Properties.Resources.exclamation_red_grey;

            Label labelZerar = new Label();
            labelZerar.Name = String.Format(Properties.Resources.labelZerarName, this._indiceTransdutores);
            labelZerar.Font = new Font(FontFamily.GenericSansSerif, 14.25F, FontStyle.Bold, GraphicsUnit.Point);
            labelZerar.ForeColor = Color.RoyalBlue;
            labelZerar.Cursor = Cursors.Hand;
            labelZerar.AutoSize = true;
            labelZerar.Text = Properties.Resources.zerarString;
            labelZerar.Location = new Point(143, 96);
            labelZerar.Click += new EventHandler(labelZerar_Click);

            CheckBox checkboxInverterSinal = new CheckBox();
            checkboxInverterSinal.Text = Properties.Resources.inverterSinalString;
            checkboxInverterSinal.Location = new Point(6, 98);
            checkboxInverterSinal.Checked = conexao.CondicionadorSinal.Calibracao.InverterSinal;
            checkboxInverterSinal.CheckedChanged += new EventHandler(checkboxInverterSinal_CheckedChanged);

            Label labelInstrumento = new Label();
            labelInstrumento.Name = String.Format(Properties.Resources.labelInstrumentoName, this._indiceTransdutores);
            labelInstrumento.Location = new Point(6, 123);
            labelInstrumento.Text = String.Format(Properties.Resources.usandoInstrumentoString, conexao.Instrumento.Descricao);
            labelInstrumento.Font = new System.Drawing.Font(FontFamily.GenericSansSerif, 7);
            labelInstrumento.AutoSize = true;
            labelInstrumento.Enabled = false;

            groupBox.Controls.Add(labelInstrumento);
            groupBox.Controls.Add(checkboxInverterSinal);
            groupBox.Controls.Add(labelZerar);
            groupBox.Controls.Add(pictureBoxAlerta);
            groupBox.Controls.Add(labelIndicacaoTransdutor);
            groupBox.Controls.Add(progressBar);
            groupBox.Controls.Add(pictureBoxBotaoConectar);
            groupBox.Controls.Add(labelConectar);
            return groupBox;
        }

        private void populaPainelTransdutores()
        {
            this.groupBoxTransdutores.Controls.Clear();
            this._dicionarioConexaoGroupTransdutores.Clear();
            this._dicionarioGroupTransdutoresConexao.Clear();

            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes())
            {
                GroupBox groupBoxTransdutor = this.criaNovoGroupBoxTransdutor(conexao);
                this._dicionarioConexaoGroupTransdutores.Add(conexao, groupBoxTransdutor);
                this._dicionarioGroupTransdutoresConexao.Add(groupBoxTransdutor, conexao);
                this._dicionatioGroupTransdutoresIndice.Add(groupBoxTransdutor, this._indiceTransdutores);
                this.groupBoxTransdutores.Controls.Add(groupBoxTransdutor);
                this.atualizaLayoutTransdutor(conexao);
                this._indiceTransdutores++;
            }
        }

        private void atualizaPainelTransdutores()
        {
            this.criaListenersConexoes();
            this.populaPainelTransdutores();
        }

        private void criaListenersConexoes()
        {
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes())
            {
                conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(conexaoChanged);
                conexao.IndicacaoChanged += new Conexao.IndicacaoChangedEvent(indicacaoChanged);
                conexao.InstrumentoChanged += new Conexao.InstrumentoChangedEvent(instrumentoChanged);
            }
        }

        private void listaConexoesChanged(object sender, GerenciadorEnsaios.ListaConexoesEventArgs e)
        {
            this.atualizaPainelTransdutores();
        }

        #endregion

        #region Atualizacao dos transdutores

        private void instrumentoChanged(object sender, Conexao.InstrumentoChangedEventArgs e)
        {
            Conexao conexao = (Conexao)sender;
            this.atualizaLayoutTransdutor(conexao);
        }

        private void indicacaoChanged(object sender, TestView.Indicacao.Indicador.IndicacaoEventArgs e)
        {
            Conexao conexao = (Conexao)sender;
            if (conexao.Conectado)
            {
                this.atualizaLayoutTransdutor(conexao, e);
            }
        }

        private void conexaoChanged(object sender, Conexao.ConexaoEventArgs e)
        {
            Conexao conexao = (Conexao)sender;
            this.atualizaLayoutTransdutor(conexao);
        }

        private void atualizaLayoutTransdutor(Conexao conexao)
        {
            GroupBox groupBoxTransdutor = this._dicionarioConexaoGroupTransdutores[conexao];
            int indice = this._dicionatioGroupTransdutoresIndice[groupBoxTransdutor];

            bool conectado = conexao.Conectado;

            Label labelConectar = (Label) groupBoxTransdutor.Controls[String.Format(Properties.Resources.labelConectarTransdutorName, indice)];
            Label labelInstrumento = (Label) groupBoxTransdutor.Controls[String.Format(Properties.Resources.labelInstrumentoName, indice)];
            PictureBox pictureBoxConectar = (PictureBox)groupBoxTransdutor.Controls[String.Format(Properties.Resources.pictureBoxConectarTransdutorName, indice)];
            Label labelIndicacaoTransdutor = (Label)groupBoxTransdutor.Controls[String.Format(Properties.Resources.labelIndicacaoTransdutorName, indice)];
            ProgressBar progressBarIndicacaoTransdutor = (ProgressBar)groupBoxTransdutor.Controls[String.Format(Properties.Resources.progressBarTransdutorName, indice)];
            PictureBox pictureBoxAlerta = (PictureBox)groupBoxTransdutor.Controls[String.Format(Properties.Resources.pictureBoxAlertaName, indice)];
            
            String textoLabelInstrumento = String.Format(Properties.Resources.usandoInstrumentoString, conexao.Instrumento.Descricao);
            String textoGroupbox = TestView.Condicionador.Calibracao.dicionarioTipoTransdutor()[conexao.CondicionadorSinal.Calibracao.TipoTransdutor];
            setGroupboxDetails(groupBoxTransdutor, textoGroupbox);

            if (conectado)
            {
                this.setLabelDetails(labelConectar, Properties.Resources.desconectarString);
                this.setPictureBoxDetails(pictureBoxConectar, Properties.Resources.connect_green);
            }
            else
            {

                this.setLabelDetails(labelConectar, Properties.Resources.conectarString);
                this.setPictureBoxDetails(pictureBoxConectar, Properties.Resources.connect_grey);
                this.setLabelDetails(labelIndicacaoTransdutor, Properties.Resources.indicacaoVaziaString);
                this.setProgressBarDetails(progressBarIndicacaoTransdutor, 0);
                this.setPictureBoxDetails(pictureBoxAlerta, null);
            }
            this.setLabelDetails(labelInstrumento, textoLabelInstrumento);
        }

        private void atualizaLayoutTransdutor(Conexao conexao, TestView.Indicacao.Indicador.IndicacaoEventArgs eventArgs)
        {
            GroupBox groupBoxTransdutor = this._dicionarioConexaoGroupTransdutores[conexao];
            int indice = this._dicionatioGroupTransdutoresIndice[groupBoxTransdutor];

            bool conectado = conexao.Conectado;
            Unidade unidade = conexao.CondicionadorSinal.Calibracao.Unidade;

            Label labelIndicacaoTransdutor = (Label)groupBoxTransdutor.Controls[String.Format(Properties.Resources.labelIndicacaoTransdutorName, indice)];
            ProgressBar progressBarIndicacaoTransdutor = (ProgressBar)groupBoxTransdutor.Controls[String.Format(Properties.Resources.progressBarTransdutorName, indice)];
            PictureBox pictureBoxAlerta = (PictureBox)groupBoxTransdutor.Controls[String.Format(Properties.Resources.pictureBoxAlertaName, indice)];

            #region Groupbox

            String textoGroupbox = TestView.Condicionador.Calibracao.dicionarioTipoTransdutor()[conexao.CondicionadorSinal.Calibracao.TipoTransdutor];

            #endregion

            #region Indicacao
            string indicacao = String.Format("{0} {1}", eventArgs.IndicacaoAjustadaStr, unidade);
            #endregion

            #region ProgressBar
            double progressBarValor = eventArgs.IndicacaoRelativa;
            #endregion

            #region Alerta

            Image imagemAlerta = null;
            if (eventArgs.IndicacaoRelativa > 0.9)
            {
                imagemAlerta = Properties.Resources.exclamation_red;
            }

            #endregion

            if (conectado)
            {
                setGroupboxDetails(groupBoxTransdutor, textoGroupbox);
                setLabelDetails(labelIndicacaoTransdutor, indicacao);
                setValorRelativoProgressBar(progressBarIndicacaoTransdutor, progressBarValor);
                setPictureBoxDetails(pictureBoxAlerta, imagemAlerta);
            }
        }

        #endregion

        #endregion

        #region Abas de ensaio

        private void moduloEnsaioAdicionado(object sender, GerenciadorEnsaios.EnsaioAdicionadoEventArgs e)
        {
            IModuloDeEnsaio moduloEnsaio = e.ModuloEnsaio;
            TabPage novaAba = new TabPage();
            novaAba.Text = moduloEnsaio.RotinaDeEnsaio.Ensaio.ToString();
            novaAba.Controls.Add(moduloEnsaio.FachadaGUI.PainelEnsaio);
            novaAba.Tag = moduloEnsaio;
            tabControlEnsaios.TabPages.Add(novaAba);
            tabControlEnsaios.SelectedTab = novaAba;
        }

        private void moduloEnsaioRemovido(object sender, GerenciadorEnsaios.EnsaioRemovidoEventArgs e)
        {
            foreach (TabPage aba in this.tabControlEnsaios.TabPages)
            {
                IModuloDeEnsaio moduloEnsaio = (IModuloDeEnsaio)aba.Tag;
                if (moduloEnsaio.Equals(e.ModuloEnsaio))
                {
                    this.tabControlEnsaios.TabPages.Remove(aba);
                    break;
                }
            }
        }

        private void adicionaMenusEnsaio(IModuloDeEnsaio moduloEnsaio)
        {
            this._menusEnsaio.AddRange(moduloEnsaio.FachadaGUI.Menus);
            foreach (ToolStripMenuItem menu in this._menusEnsaio)
            {
                this.menuStripPrincipal.Items.Add(menu);
            }
        }

        private void selecionaAbaEnsaio(int direcao)
        {
            if (this.tabControlEnsaios.SelectedTab != null)
            {
                int novoIndice = (this.tabControlEnsaios.TabCount + this.tabControlEnsaios.SelectedIndex + direcao) % this.tabControlEnsaios.TabCount;
                this.tabControlEnsaios.SelectedIndex = novoIndice;
            }
        }

        private void fecharAbaEnsaio()
        {
            if (this.tabControlEnsaios.SelectedTab != null)
            {
                TabPage aba = this.tabControlEnsaios.SelectedTab;
                IModuloDeEnsaio moduloEnsaio = (IModuloDeEnsaio)aba.Tag;
                GerenciadorEnsaios.getInstance().removeEnsaio(moduloEnsaio);
            }
        }

        private void removeMenusEnsaio()
        {
            foreach (ToolStripMenuItem menu in this._menusEnsaio)
            {
                this.menuStripPrincipal.Items.Remove(menu);
            }
            this._menusEnsaio.Clear();
        }

        private void tabControlEnsaiosSelected(object sender, TabControlEventArgs e)
        {
            this.atualizaMenusDeEnsaio();
        }

        private void atualizaMenusDeEnsaio()
        {
            lock (this.tabControlEnsaios)
            {
                this.removeMenusEnsaio();
                TabPage abaSelecionada = this.tabControlEnsaios.SelectedTab;
                if (abaSelecionada != null)
                {
                    IModuloDeEnsaio moduloEnsaio = (IModuloDeEnsaio)abaSelecionada.Tag;
                    this.adicionaMenusEnsaio(moduloEnsaio);
                }
            }
        }

        private void tabControlEnsaios_ControlAdded(object sender, ControlEventArgs e)
        {
            if (this.tabControlEnsaios.TabPages.Count == 1)
            {
                this.atualizaMenusDeEnsaio();
            }
        }

        #endregion

        #region Ensaio

        private void iniciaEnsaio()
        {
            if (tabControlEnsaios.SelectedTab != null)
            {
                IModuloDeEnsaio moduloEnsaio = (IModuloDeEnsaio)tabControlEnsaios.SelectedTab.Tag;
                moduloEnsaio.RotinaDeEnsaio.iniciaEnsaioEspecifico();
            }
        }

        #endregion

        #region Delegates para atualização do layout

        private delegate void SetLabelDetailsCallBack(Label label, string texto);

        private void setLabelDetails(Label label, string texto)
        {
            try
            {
                if (label.InvokeRequired)
                {
                    SetLabelDetailsCallBack d = new SetLabelDetailsCallBack(setLabelDetails);
                    this.Invoke(d, new object[] { label, texto });
                }
                else
                {
                    label.Text = texto;
                }
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(ex.Message, ex, false);
            }
        }

        private delegate void SetProgressBarDetailsCallBack(ProgressBar progressBar, int valor);

        private void setProgressBarDetails(ProgressBar progressBar, int valor)
        {
            try
            {
                if (progressBar.InvokeRequired)
                {
                    SetProgressBarDetailsCallBack d = new SetProgressBarDetailsCallBack(setProgressBarDetails);
                    this.Invoke(d, new object[] { progressBar, valor });
                }
                else
                {
                    progressBar.Value = valor;
                }
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(ex.Message, ex, false);
            }
        }

        private delegate void SetValorRelativoProgressBarCallBack(ProgressBar progressBar, double valorRelativo);

        private void setValorRelativoProgressBar(ProgressBar progressBar, double valorRelativo)
        {
            try
            {
                if (progressBar.InvokeRequired)
                {
                    SetValorRelativoProgressBarCallBack d = new SetValorRelativoProgressBarCallBack(setValorRelativoProgressBar);
                    this.Invoke(d, new object[] { progressBar, valorRelativo });
                }
                else
                {
                    int progressBarValor = Convert.ToInt32(Math.Abs(progressBar.Maximum * valorRelativo));
                    progressBarValor = Math.Min(progressBarValor, progressBar.Maximum);
                    progressBar.Value = progressBarValor;
                }
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(ex.Message, ex, false);
            }
        }

        private delegate void SetPictureBoxDetailsCallBack(PictureBox pictureBox, Image imagem);

        private void setPictureBoxDetails(PictureBox picturebox, Image imagem)
        {
            try
            {
                if (picturebox.InvokeRequired)
                {
                    SetPictureBoxDetailsCallBack d = new SetPictureBoxDetailsCallBack(setPictureBoxDetails);
                    this.Invoke(d, new object[] { picturebox, imagem });
                }
                else
                {
                    picturebox.Image = imagem;
                }
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(ex.Message, ex, false);
            }
        }

        private delegate void SetGroupboxDetailsCallBack(GroupBox groupbox, String texto);

        private void setGroupboxDetails(GroupBox groupbox, String texto)
        {
            try
            {
                if (groupbox.InvokeRequired)
                {
                    SetGroupboxDetailsCallBack d = new SetGroupboxDetailsCallBack(setGroupboxDetails);
                    this.Invoke(d, new object[] { groupbox, texto });
                }
                else
                {
                    groupbox.Text = texto;
                }
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(ex.Message, ex, false);
            }
        }

        #endregion

        #region Menu novo ensaio

        private void bibliotecasCarregadasChanged(object sender, CarregadorDeModulos.BibliotecasCarregadasEventArgs e)
        {
            this.atualizaMenuNovoEnsaio(e.BibliotecasCarregadas);
        }

        private void atualizaMenuNovoEnsaio()
        {
            this.atualizaMenuNovoEnsaio(CarregadorDeModulos.getInstance().BibliotecasCarregadas);
        }

        private void atualizaMenuNovoEnsaio(List<BibliotecaModulo> bibliotecas)
        {
            menuNovoEnsaio.DropDownItems.Clear();
            foreach (BibliotecaModulo modulo in bibliotecas)
            {
                ToolStripItem novoEnsaio = new ToolStripMenuItem();
                novoEnsaio.Tag = modulo;
                novoEnsaio.Text = modulo.NomeEnsaio;
                novoEnsaio.Click += new EventHandler(menuNovoEnsaio_Click);
                menuNovoEnsaio.DropDownItems.Add(novoEnsaio);
            }
        }

        #endregion

        #region Abrir/Salvar ensaio

        private void salvarEnsaioEmArquivo()
        {
            if (tabControlEnsaios.SelectedTab != null)
            {
                SaveFileDialog dialogoSalvarArquivo = new SaveFileDialog();
                if (dialogoSalvarArquivo.ShowDialog(this) == DialogResult.OK)
                {
                    String nomeArquivo = dialogoSalvarArquivo.FileName;
                    IModuloDeEnsaio moduloEnsaio = (IModuloDeEnsaio)tabControlEnsaios.SelectedTab.Tag;
                    GerenciadorEnsaios.getInstance().salvarArquivoEnsaio(nomeArquivo, moduloEnsaio);
                }
            }
        }

        private void abrirEnsaioDoArquivo()
        {
            OpenFileDialog dialogoAbrirArquivo = new OpenFileDialog();
            if (dialogoAbrirArquivo.ShowDialog(this) == DialogResult.OK)
            {
                String nomeArquivo = dialogoAbrirArquivo.FileName;
                GerenciadorEnsaios.getInstance().abrirArquivoDeEnsaio(nomeArquivo);
            }
        }

        #endregion

        #region Eventos do form

        private void menuSair_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void menuConexoes_Click(object sender, EventArgs e)
        {
            FormConexoes formConexoes = new FormConexoes();
            formConexoes.ShowDialog(this);
        }

        private void calibraçãoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormCalibracao formCalibracao = new FormCalibracao();
            formCalibracao.ShowDialog(this);
        }

        private void FormPrincipal_FormClosing(object sender, FormClosingEventArgs e)
        {
            GerenciadorEnsaios.getInstance().encerrar();
        }

        private void arquivosDeCalibraçãoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormArquivosDeCalibracao formArquivosDeCalibracao = new FormArquivosDeCalibracao();
            formArquivosDeCalibracao.ShowDialog(this);
        }

        private void pictureBoxBotaoConectar_Click(object sender, EventArgs e)
        {
            Control pictureBoxConectar = (Control)sender;
            GroupBox groupBox = (GroupBox) pictureBoxConectar.Parent;
            Conexao conexao = this._dicionarioGroupTransdutoresConexao[groupBox];
            GerenciadorEnsaios.getInstance().alternarEstadoConexao(conexao);
        }

        private void labelConectar_Click(object sender, EventArgs e)
        {
            Control labelConectar = (Control)sender;
            GroupBox groupBox = (GroupBox)labelConectar.Parent;
            Conexao conexao = this._dicionarioGroupTransdutoresConexao[groupBox];
            GerenciadorEnsaios.getInstance().alternarEstadoConexao(conexao);
        }

        private void labelZerar_Click(object sender, EventArgs e)
        {
            Control labelZerar = (Control)sender;
            GroupBox groupBox = (GroupBox)labelZerar.Parent;
            Conexao conexao = this._dicionarioGroupTransdutoresConexao[groupBox];
            GerenciadorEnsaios.getInstance().zerar(conexao);
        }

        private void checkboxInverterSinal_CheckedChanged(object sender, EventArgs e)
        {
            Control checkboxInverterSinal = (Control)sender;
            GroupBox groupBox = (GroupBox)checkboxInverterSinal.Parent;
            Conexao conexao = this._dicionarioGroupTransdutoresConexao[groupBox];
            GerenciadorEnsaios.getInstance().inverterSinal(conexao);
        }

        private void exibirTransdutoresToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.alternarPainelTransdutores();
        }

        private void splitContainerPrincipal_Resize(object sender, EventArgs e)
        {
            if (splitContainerPrincipal.Width > 0)
            {
                int newDistance = splitContainerPrincipal.Width - 230;
                newDistance = Math.Max(0, newDistance);
                splitContainerPrincipal.SplitterDistance = newDistance;
            }
        }

        private void menuNovoEnsaio_Click(object sender, EventArgs e)
        {
            ToolStripItem controle = (ToolStripItem)sender;
            BibliotecaModulo modulo = (BibliotecaModulo) controle.Tag;
            GerenciadorEnsaios.getInstance().novoEnsaio(modulo);
        }

        private void toolStripMenuItemFecharAba_Click(object sender, EventArgs e)
        {
            this.fecharAbaEnsaio();
        }

        private void ocultarTransdutoresToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.exibirTransdutoresToolStripMenuItem.Checked = !this.exibirTransdutoresToolStripMenuItem.Checked;
        }

        private void menuAbrirEnsaioDoArquivo_Click(object sender, EventArgs e)
        {
            this.abrirEnsaioDoArquivo();
        }

        private void menuSalvarEnsaioEmArquivo_Click(object sender, EventArgs e)
        {
            this.salvarEnsaioEmArquivo();
        }

        private void FormPrincipal_KeyDown(object sender, KeyEventArgs e)
        {
            Keys[] teclasNumericas = new Keys[] {Keys.D1, Keys.D2, Keys.D3, Keys.D4, Keys.D5, Keys.D6, Keys.D7, Keys.D8, Keys.D9};
            if (e.Control)
            {
                if (teclasNumericas.Contains(e.KeyCode)) {
                    int indiceConexao = e.KeyCode - Keys.D1;
                    this.alternaEstadoConexao(indiceConexao);
                }
                else if (e.KeyCode == Keys.Space)
                {
                    this.iniciaEnsaio();
                }
                else if (e.KeyCode == Keys.X)
                {
                    this.fecharAbaEnsaio();
                }
                else if (e.KeyCode == Keys.A)
                {
                    this.abrirEnsaioDoArquivo();
                }
                else if (e.KeyCode == Keys.S)
                {
                    this.salvarEnsaioEmArquivo();
                }
                else if (e.KeyCode == Keys.H)
                {
                    this.alternarPainelTransdutores();
                }
                else if (e.KeyCode == Keys.Down)
                {
                    this.selecionaAbaEnsaio(1);
                }
                else if (e.KeyCode == Keys.Up)
                {
                    this.selecionaAbaEnsaio(-1);
                }
            }
        }

        private void instrumentaçãoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormInstrumentacao form = new FormInstrumentacao();
            form.ShowDialog();
        }

        #endregion

        private void bancoDeDadosToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormBancoDados form = new FormBancoDados();
            form.ShowDialog();
        }
    }
}
