﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using TestView.Ensaios;
using TestView.Seguranca;

namespace TestViewGUI.Relatorios
{
    public partial class FormOpcoesImpressao : Form
    {
        #region Construtor e membros

        private IModuloDeEnsaio _modulo;
        private PrintDocument _documento;

        public FormOpcoesImpressao(IModuloDeEnsaio modulo)
        {
            InitializeComponent();
            this._modulo = modulo;
            this._documento = new PrintDocument();
            this._documento.DefaultPageSettings.Landscape = true;
            this._documento.PrintPage += new PrintPageEventHandler(document_PrintPage);
            this.inicializaLayout();
        }

        #endregion

        #region Layout

        private void inicializaLayout()
        {
            this.inicializaComboboxImpressoras();
        }

        private void inicializaComboboxImpressoras()
        {
            this.comboBoxImpressoras.Items.Clear();
            PrinterSettings configImpressora = new PrinterSettings();
            foreach (String nomeImpressora in PrinterSettings.InstalledPrinters)
            {
                this.comboBoxImpressoras.Items.Add(nomeImpressora);
                configImpressora.PrinterName = nomeImpressora;
                if (configImpressora.IsDefaultPrinter)
                {
                    this.comboBoxImpressoras.SelectedItem = nomeImpressora;
                }
            }
        }

        #endregion

        #region Impressao

        private void imprime()
        {
            try
            {
                this._documento.Print();
            }
            catch (Exception ex)
            {
                Logging.getInstance().warn(Properties.Resources.impossivelImprimirRelatorioString, ex, true);
            }
        }

        private void document_PrintPage(object sender, PrintPageEventArgs e)
        {
            this._modulo.RotinaDeEnsaio.imprimeRelatorio(e);
        }

        #endregion

        #region Eventos

        private void comboBoxImpressoras_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.comboBoxImpressoras.SelectedItem != null)
            {
                this._documento.PrinterSettings.PrinterName = (String)this.comboBoxImpressoras.SelectedItem;
            }
        }

        private void buttonImprimir_Click(object sender, EventArgs e)
        {
            this.imprime();
            this.Close();
        }

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}
